import copy


class PhDBAPI(object):
    engine = None
    session = None

    def tables(self):
        """
        列出全部数据表
        :return:
        """
        return self.engine.table_names()

    def query(self, obj, **ext):
        """
        查询数据
        :param obj: 要查询的实例信息
        :param ext: 额外查询条件，如查询空值，如 name=None
        :return:
        """
        result = self.session.query(obj.__class__)
        for k, v in obj.__dict__.items():
            if k != '_sa_instance_state' and v:
                result = result.filter(getattr(obj.__class__, k) == v)
        for k, v in ext.items():
            result = result.filter(getattr(obj.__class__, k) == v)
        result = result.all()
        return result

    def insert(self, obj):
        """
        插入数据
        :return:
        """
        self.session.add(obj)
        self.session.flush()
        return obj

    def commit(self):
        self.session.commit()

    def delete(self, obj):
        result = self.query(obj)
        for r in result:
            self.session.delete(r)
        return result

    def update(self, obj, FK='id'):
        """
        更新数据
        :param obj: 要更新的实例信息
        :param FK: 主键
        :return:
        """
        tmp = copy.deepcopy(obj.__dict__)
        tmp.pop('_sa_instance_state', None)
        obj_id = tmp.pop(FK, None)

        # 如果没有要更新的元素，直接返回
        if not obj_id or not tmp:
            return obj

        result = self.session.query(obj.__class__).filter(getattr(obj.__class__, FK) == obj_id)
        result.update(tmp)
        return obj



# -*- coding: utf-8 -*-


class ColCharactor(object):
    """
    对于每个单元格的匹配规则
    """

    def __init__(self, col_name: str, col_desc: str, candidate: list = [], type: str = 'String',
                 not_null: bool = False):
        self.col_name = col_name
        self.col_desc = col_desc
        self.candidate = candidate
        self.type = type
        self.not_null = not_null

    def to_dict(self) -> dict:
        return self.__dict__


class DataMapping(object):
    """
    对于指定源的指定公司的匹配规则
    """

    def __init__(self, source: str, company: str, cols: list):
        self.source = source
        self.company = company
        self.cols = cols

    def to_dict(self) -> dict:
        self.__dict__['cols'] = [col.to_dict() for col in self.cols]
        return self.__dict__

    def get_metadata(self) -> list:
        return [{'key': col.col_name, 'type': col.type} for col in self.cols]





from datetime import datetime
from sqlalchemy import Column, String, INTEGER, BLOB, FLOAT
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()


class Dag(Base):
    __tablename__ = 'dag'

    dag_id = Column(String, primary_key=True)
    is_paused = Column(INTEGER)
    is_subdag = Column(INTEGER)
    is_active = Column(INTEGER)
    last_scheduler_run = Column(String, default=datetime.now())
    last_pickled = Column(String, default=datetime.now())
    last_expired = Column(String, default=datetime.now())
    scheduler_lock = Column(INTEGER)
    pickle_id = Column(INTEGER)
    fileloc = Column(String)
    owners = Column(String)
    description = Column(String)
    default_view = Column(String)
    schedule_interval = Column(String)
    root_dag_id = Column(String)
    next_dagrun = Column(String, default=datetime.now())
    next_dagrun_create_after = Column(String, default=datetime.now())
    concurrency = Column(INTEGER)
    has_task_concurrency_limits = Column(INTEGER)

    def __repr__(self):
        return str(self.__dict__)


class DagRun(Base):
    __tablename__ = 'dag_run'

    id = Column(String, primary_key=True)
    dag_id = Column(String)
    execution_date = Column(String, default=datetime.now())
    state = Column(String)
    run_id = Column(String)
    external_trigger = Column(INTEGER)
    conf = Column(BLOB)
    end_date = Column(String, default=datetime.now())
    start_date = Column(String, default=datetime.now())
    run_type = Column(String)
    last_scheduling_decision = Column(String, default=datetime.now())
    dag_hash = Column(String)
    creating_job_id = Column(INTEGER)

    def __repr__(self):
        return str(self.__dict__)


class TaskInstance(Base):
    __tablename__ = 'task_instance'

    task_id = Column(String, primary_key=True)
    dag_id = Column(String, primary_key=True)
    execution_date = Column(String, default=datetime.now(), primary_key=True)
    start_date = Column(String, default=datetime.now())
    end_date = Column(String, default=datetime.now())
    duration = Column(FLOAT)
    state = Column(String)
    try_number = Column(INTEGER)
    hostname = Column(String)
    unixname = Column(String)
    job_id = Column(INTEGER)
    pool = Column(String)
    queue = Column(String)
    priority_weight = Column(INTEGER)
    operator = Column(String)
    queued_dttm = Column(String, default=datetime.now())
    pid = Column(INTEGER)
    max_tries = Column(INTEGER)
    executor_config = Column(BLOB)
    pool_slots = Column(INTEGER)
    queued_by_job_id = Column(INTEGER)
    external_executor_id = Column(String)

    def __repr__(self):
        return str(self.__dict__)



# -*- coding: utf-8 -*-

from phcli.ph_lmd.runtime.lambda_rt import LambdaRuntime


class NodejsRT(LambdaRuntime):
    __runtime_name = "nodejs"

    def pkg_layer(self, data):
        """
        组织 layer 的打包逻辑
        :param data
            :arg lib_path: 指定 nodejs 的 lib 位置
            :arg package_name: 打包的名称
        """

        self._package_cmds = [
            "rm -rf %s" % (data["lib_path"]),
            "npm install --production",
            "mkdir -p %s/%s/node_modules" % (self._package_root, self.__runtime_name),
            "cp -r %s/* %s/%s/node_modules/" % (data["lib_path"], self._package_root, self.__runtime_name),
            "cd %s && zip -r -q ../package.zip . && cd -" % self._package_root,
            "rm -rf %s" % self._package_root,
        ]

        self.package(data["package_name"])

    def pkg_code(self, data):
        """
        组织 code 的打包逻辑
        :param data
            :arg name: 项目名称
            :arg code_path: 需要打包的代码位置
            :arg package_name: 打包的名称
        """

        self._package_cmds = [
            "mkdir -p %s/%s" % (self._package_root, self.__runtime_name),
            "npm i",
            "npm run build",
        ]

        for path in data["code_path"].split(","):
            if path.endswith("/"):
                path = path[:-1]
            self._package_cmds.append("cp -r %s %s/%s/" % (path, self._package_root, self.__runtime_name))

        if "name" in data.keys():
            self._package_cmds.extend([
                "mv %s/%s/config/project/%s.yml %s/%s/config/server.yml" % (self._package_root, self.__runtime_name, data['name'], self._package_root, self.__runtime_name),
                "rm -rf %s/%s/config/project/" % (self._package_root, self.__runtime_name)
            ])

        self._package_cmds.extend([
            "cd %s/%s && zip -r -q ../../package.zip . && cd -" % (self._package_root, self.__runtime_name),
            "rm -rf %s" % self._package_root,
        ])

        self.package(data["package_name"])



# -*- coding: utf-8 -*-

def universe_blue_grey_mapping():
    return [
        {
            "blue_col_name": "PHA_ID",
            "grey_col_name": ["3#PHA ID", "3#PHA_ID"],
        },
        {
            "blue_col_name": "PHA_HOSP_NAME",
            "grey_col_name": ["2#PHA Hospname", "2#PHA_Hospname"],
        },
        {
            "blue_col_name": "SPECIALTY_CATE",
            "grey_col_name": ["29#Specialty_1", ],
        },
        {
            "blue_col_name": "SPECIALTY_ADMIN",
            "grey_col_name": ["30#Specialty_2", ],
        },
    ]


def universe_blue_orange_mapping():
    return [
        {
            "blue_col_name": "PHA_HOSP_NAME",
            "orange_col_name": ["63#医院名称", ],
        },
        {
            "blue_col_name": "PROVINCE",
            "orange_col_name": ["64#省份", ],
        },
        {
            "blue_col_name": "CITY",
            "orange_col_name": ["65#城市", ],
        },
        {
            "blue_col_name": "DISTRICT",
            "orange_col_name": ["66#区/县", ],
        },
        {
            "blue_col_name": "HOSP_LEVEL",
            "orange_col_name": ["67#医院级别", ],
        },
        {
            "blue_col_name": "HOSP_QUALITY",
            "orange_col_name": ["70#公立/私立", ],
        },
        {
            "blue_col_name": "SPECIALTY_CATE",
            "orange_col_name": ["68#综合/专科", ],
        },
        {
            "blue_col_name": "SPECIALTY_ADMIN",
            "orange_col_name": ["69#专科类别", ],
        },
        {
            "blue_col_name": "DOCTORS_NUM",
            "orange_col_name": ["72#医生数", ],
        },
        {
            "blue_col_name": "ANNU_DIAG_TIME",
            "orange_col_name": ["78#年诊疗人次", ],
        },
        {
            "blue_col_name": "OUTP_DIAG_TIME",
            "orange_col_name": ["79#门诊诊次", ],
        },
    ]



# -*- coding: utf-8 -*-

"""
递归获取 s3 中所有的目录，保留结构
"""

import boto3
import xlwt

client = boto3.client('s3')

buckets = client.list_buckets()
buckets = [bucket['Name'] for bucket in buckets['Buckets']]


def transition_tree(tree, k, str):
    # 终止条件
    if not str or '/' not in str:
        tree[k] = {}
        return tree

    # 初始化 key
    if k not in tree.keys():
        tree[k] = {}

    sub_tree = tree[k]
    sub_k, sub_str = str.split('/', 1)
    sub_result = transition_tree(sub_tree, sub_k, sub_str)
    sub_tree.update(sub_result)

    tree[k] = sub_tree
    return tree


main_tree = {}
for bucket in buckets:
    objects = client.list_objects(
        Bucket=bucket
    )
    objects = [object['Key'] for object in objects['Contents']]

    tree = {}
    for object in objects:
        if '/' not in object:
            continue
        sub_k, sub_str = object.split('/', 1)
        transition_tree(tree, sub_k, sub_str)
    main_tree[bucket] = tree


row, col = 0, 0
def write_excel_xls_by_subtree(sheet, sub_tree):
    if not sub_tree:
        return

    global row, col
    for k, v in sub_tree.items():
        print(row, col, k)
        sheet.write(row, col, k)
        row += 1
        col += 1
        write_excel_xls_by_subtree(sheet, v)
        col -= 1


workbook = xlwt.Workbook()
sheet = workbook.add_sheet('目录情况')

for k, v in main_tree.items():
    write_excel_xls_by_subtree(sheet, {k: v})

workbook.save('s3当前目录情况.xls')
print("xls格式表格写入数据成功！")



import click
from phcli.ph_admin import pg
from phcli.ph_admin.ph_models import Scope, Role


@click.group("scope", short_help='权限管理工具')
def main():
    pass


@click.command("create", short_help='创建权限')
@click.option("-n", "--name", help="权限名", prompt="权限名")
@click.option("-d", "--description", help="权限描述", prompt="权限描述")
@click.option("-s", "--scopePolicy", help="权限策略", prompt="权限策略")
@click.option("-o", "--owner", multiple=True, help="附加角色")
def create_scope(**kwargs):
    kwargs['scopePolicy'] = kwargs.pop('scopepolicy')

    owners = []
    for owner in kwargs['owner']:
        for owner in pg.query(Role(), name=owner):
            owners.append(owner.id)
    kwargs['owner'] = owners

    result = pg.insert(Scope(**kwargs))
    click.secho('添加成功'+str(result), fg='green', blink=True, bold=True)
    pg.commit()


@click.command("update", short_help='更新权限')
def update_scope():
    click.secho('未实现', fg='red', blink=True, bold=True)
    pass


@click.command("list", short_help='列举权限')
def list_scope():
    for scope in pg.query(Scope()):
        click.secho(str(scope), fg='green', blink=True, bold=True)


@click.command("get", short_help='查找权限')
@click.option("-n", "--name", help="权限名", default=None)
def get_scope(**kwargs):
    for scope in pg.query(Scope(**kwargs)):
        click.secho(str(scope), fg='green', blink=True, bold=True)


@click.command("delete", short_help='删除权限')
@click.option("-n", "--name", help="权限名", prompt="权限名")
def delete_scope(**kwargs):
    for scope in pg.delete(Scope(**kwargs)):
        click.secho(str(scope), fg='green', blink=True, bold=True)
    pg.commit()


main.add_command(create_scope)
main.add_command(update_scope)
main.add_command(list_scope)
main.add_command(get_scope)
main.add_command(delete_scope)



import click
from phcli.ph_aws_cli.init_conf.__main__ import aws_init


@click.group("aws_cli", short_help='aws_cli系列命令')
def main():
    """
    本脚本用于执行aws cli系列命令
    """
    pass


main.add_command(aws_init)


if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-

import boto3
import copy
import pandas as pd

from phcli.ph_data_clean.util.yaml_utils import append_to_file, load_by_dir


BUCKER_NAME = 'ph-origin-files'
LOCAL_CACHE_DIR = r'../../../file/ph_data_clean/s3_primitive_data/'
filter_dir = ['OTHERS']

KEEP_ROW_COUNT = 2

s3 = boto3.resource('s3')
s3_client = boto3.client('s3')


def get_all_file_path():
    """
    获取指定桶中的所有文件信息

    :return: [[filter_file_lst], [all_file_path], [err_file_path]]
        filter_file_lst: 被过滤掉的文件信息
        all_file_path: 可爬取的文件信息
        err_file_path: 无法解析路径的文件信息
    """
    filter_file_lst = []
    all_file_lst = []
    err_file_lst = []

    bucket = s3.Bucket(BUCKER_NAME)
    for obj in bucket.objects.all():
        obj_key = obj.key
        obj_key_lst = obj_key.split("/")

        # 目录过滤掉
        if obj_key.endswith("/"):
            continue

        # 不需要的目录过滤掉
        if obj_key_lst[0] in filter_dir:
            filter_file_lst.append(obj_key)
            continue

        # 正常处理获取文件信息
        if len(obj_key_lst) == 3 and obj_key_lst[2].endswith(".xlsx") or obj_key_lst[2].endswith(".xls"):
            all_file_lst.append({
                "source": repr(obj_key_lst[0]).strip("'"),
                "company": repr(obj_key_lst[1]).strip("'"),
                "file": repr(obj_key).strip("'"),
            })
            continue

        # 如果到这里，证明文件路径问题，加入错误流程
        err_file_lst.append(obj_key)

    return filter_file_lst, all_file_lst, err_file_lst


def load_cache_data():
    """
     获取本地缓存的文件信息

     :return: cache_data_lst: 已经处理的文件信息
     """
    return load_by_dir(LOCAL_CACHE_DIR)


def append_cache_data(parse_sheet_lst):
    """
    向文件中追加新解析的文件信息

    :param parse_file_lst: 解析成功的页信息
    """
    if len(parse_sheet_lst):
        append_to_file(parse_sheet_lst, f'''{LOCAL_CACHE_DIR}/{parse_sheet_lst[0]["source"]}-{parse_sheet_lst[0]["company"]}.yaml''')


def get_s3_increment(cache_data_lst, s3_all_file_lst):
    """
    获取 s3 上的增量文件，对于同名但重新上传的文件暂不支持重新处理

    :param cache_data_lst: 缓存的文件列表
    :param s3_all_file_lst: s3 上所有可解析的文件列表

    :return: increment_file_lst: 增量文件列表
    """
    # 利用 map 去重当前缓存的所有 file
    cache_file_map = {}
    for cache_data in cache_data_lst:
        source = cache_data['source']
        company = cache_data['company']

        # 记录 source
        if source not in cache_file_map.keys():
            cache_file_map[source] = {}

        # 记录 company
        if company not in cache_file_map[source].keys():
            cache_file_map[source][company] = []

        # 追加文件
        if cache_data['file'] not in cache_file_map[source][company]:
            # 看source里面是否存在对应company信息，若有返回对应value（即文件名.xlsx）,否则返回空list
            lst = cache_file_map[source].get(company, [])
            lst.append(cache_data['file'])
            cache_file_map[source][company] = lst  # 所有文件名储存在list里面，这个list是cache_file_map[source][company]的value

    increment_file_lst = []
    for s3_file in s3_all_file_lst:
        source = s3_file['source']
        company = s3_file['company']

        if source in cache_file_map.keys() \
                and company in cache_file_map[source].keys() \
                and s3_file['file'] in cache_file_map[source][company]:
            continue

        increment_file_lst.append(s3_file)
    return increment_file_lst


def parse_s3_execl(obj):
    """
    解析每个 execl 的全部 sheet

    :return: sheets_info: 每个 sheet 的信息
    """
    sheets_info = []
    response = s3_client.get_object(
        Bucket=BUCKER_NAME,
        Key=obj["file"],
    )
    excel_data = pd.ExcelFile(response['Body'].read())

    if not len(excel_data.sheet_names):
        cp_obj = copy.deepcopy(obj)
        cp_obj["length"] = 0
        cp_obj["data"] = {}
        cp_obj["err"] = 'sheet number is 0'
        sheets_info.append(cp_obj)
        del cp_obj
        return sheets_info

    for sheet_name in excel_data.sheet_names:
        cp_obj = copy.deepcopy(obj)
        cp_obj["length"] = excel_data.parse(sheet_name).shape[0]
        cp_obj["sheet"] = sheet_name
        cp_obj["data"] = excel_data.parse(sheet_name, nrows=KEEP_ROW_COUNT).to_dict()
        sheets_info.append(cp_obj)
        del cp_obj

    return sheets_info


if __name__ == '__main__':
    cache_data_lst = load_cache_data()  # 本地已缓存的文件信息
    print(f"当前缓存了 {len(cache_data_lst)} 条数据（包含 sheet）")
    print()

    filter_file_lst, s3_all_file_lst, err_file_lst = get_all_file_path()
    if len(filter_file_lst):
        print(f"存在过滤掉的文件 {len(filter_file_lst)} 个，信息如下：")
        print()
    #
    if len(err_file_lst):
        print(f"存在无法解析路径或后缀的文件 {len(err_file_lst)} 个，信息如下：")
        print()

    increment_lst = get_s3_increment(cache_data_lst, s3_all_file_lst)
    print(increment_lst)
    if len(increment_lst):
        print(f"存在增量文件 {len(increment_lst)} 个，信息如下：")
        print()
    else:
        print(f"不存在新增文件")

    for index, obj in enumerate(increment_lst, 1):
        print(f"{index}/{len(increment_lst)} 开始解析 {obj['file']}")
        append_cache_data(parse_s3_execl(obj))



from phcli.ph_storage.model.s3_storage import PhS3Storage
from phcli.ph_storage import static as st


# 测试上传S3
def upload2S3():
    download_path = "/Users/qianpeng/Desktop/tmp"
    file_name = "面试问题.txt"
    sub_path = "tmp"
    upload_path = (st.UPLOADPATH + "/" + sub_path + "/" + file_name).replace("//", "/")
    s3 = PhS3Storage()
    s3.upload(download_path + "/" + file_name, st.BUCKET, upload_path)



# -*- coding: utf-8 -*-

import os
import yaml
import click
import base64
import time

from phcli.ph_aws.ph_sts import PhSts
from phcli.ph_aws.ph_s3 import PhS3
from phcli.ph_lmd import define_value as dv
from phcli.ph_lmd.model import ph_layer
from phcli.ph_lmd.model import ph_lambda
from phcli.ph_lmd.model import ph_gateway


@click.group("deploy")
def deploy():
    """
    自动化部署一系列 Lambda 技术栈
    """
    pass


@deploy.command("init")
@click.option('-n', '--name', prompt='项目名称', help='项目名称')
@click.option('-R', '--runtime', prompt='项目使用的运行时', help='项目使用的运行时',
              type=click.Choice(['python3.6', 'python3.8', 'nodejs10.x', 'go1.x']))
@click.option('-D', '--desc', prompt='项目描述', help='项目描述')
@click.option('-L', '--lib_path', prompt='layer 依赖目录'
                                         '(Python 如".venv/lib/python3.8/site-packages", Nodejs 如"node_modules")',
              help='layer 依赖目录')
@click.option('-C', '--code_path', prompt='function 代码目录（如 "src"）', help='function 代码目录')
@click.option('-H', '--handler', prompt='lambda function 入口', help='lambda function 入口')
def init(name, runtime, desc, lib_path, code_path, handler):
    """
    初始化环境，关联本地项目和 lambda function
    """

    phsts = PhSts().assume_role(
        base64.b64decode(dv.ASSUME_ROLE_ARN).decode(),
        dv.ASSUME_ROLE_EXTERNAL_ID,
    )
    phs3 = PhS3(phsts=phsts)

    buf = phs3.open_object(
            bucket_name=dv.DEFAULT_BUCKET,
            object_name=dv.CLI_VERSION+dv.DEFAULT_TEMPLATE_DIR+dv.DEPLOY_FILE_TEMPLATE_NAME,
        ) \
        .replace("#name#", name) \
        .replace("#runtime#", runtime) \
        .replace("#desc#", desc) \
        .replace("#lib_path#", lib_path) \
        .replace("#code_path#", code_path) \
        .replace("#handler#", handler)

    if os.path.exists(dv.DEPLOY_FILE_LOCAL_NAME):
        with open(dv.DEPLOY_FILE_LOCAL_NAME) as f:
            deploy_conf = yaml.safe_load(f)
            if name in deploy_conf.keys():
                click.secho("Init Error，name '%s' is exists" % name, fg='red', blink=True, bold=True)
                return
            f.write(buf)
            click.secho("Append Init Success", fg='green', blink=True, bold=True)
            return
    else:
        with open(dv.DEPLOY_FILE_LOCAL_NAME, "wt") as wt:
            wt.write(buf)
        click.secho("Download Init Success", fg='green', blink=True, bold=True)
        return


@deploy.command("push", short_help='发布function并自动关联到API Gateway')
@click.option('-n', '--name', prompt='指定提交的项目，如果只代理一个项目则无需传入', default='', help='指定提交的项目')
@click.option('-o', '--oper', prompt='执行操作',
              type=click.Choice(['defalut', 'all', 'lib', 'code', 'api']),
              default='defalut', help='要执行的操作')
def push(name, oper):
    """
    【请在项目的根目录执行】

    \b
    发布依赖到 lambda layer, 发布项目代码到 lambda function, 并在 API Gateway 中关联到当前 lambda function 别名

    \b
    :arg oper:
        :arg default: 默认不传参数，按照预计使用频率，所以只发布 function + gateway
        :arg all: 发布全部资源 （layer、function、gateway）
        :arg lib: 只发布 layer
        :arg code: 只发布 function
        :arg api: 只发布 gateway
    """

    def apply(deploy_conf):
        if "layer" in deploy_conf.keys():
            layer = ph_layer.PhLayer()
            args = deploy_conf["metadata"]
            args.update(deploy_conf["layer"])

            if "lib_path" in deploy_conf["layer"]:
                click.secho("开始打包本地依赖: " + deploy_conf["layer"]["lib_path"] + "\t->\t" +
                            deploy_conf["layer"]["package_name"], blink=True, bold=True)
                layer.package(args)
                click.secho("本地依赖打包完成", fg='green', blink=True, bold=True)

            response = layer.apply(args)
            click.secho("layer 更新完成: " + response["LayerVersionArn"], fg='green', blink=True, bold=True)
            click.secho()

        if "lambda" in deploy_conf.keys():
            lambda_function = ph_lambda.PhLambda()
            args = deploy_conf["metadata"]
            args.update(deploy_conf["lambda"])

            if "code_path" in deploy_conf["lambda"]:
                click.secho("开始打包本地代码: " + deploy_conf["lambda"]["code_path"] + "\t->\t" +
                            deploy_conf["lambda"]["package_name"], blink=True, bold=True)
                lambda_function.package(args)
                click.secho("本地代码打包完成", fg='green', blink=True, bold=True)

            response = lambda_function.apply(args)
            click.secho("lambda 更新完成: " + response["AliasArn"], fg='green', blink=True, bold=True)
            click.secho()

        if "gateway" in deploy_conf.keys():
            gateway = ph_gateway.PhGateway()
            args = deploy_conf["metadata"]
            args.update(deploy_conf["gateway"])
            response = gateway.apply(args)
            click.secho("gateway 更新完成: " + response, fg='green', blink=True, bold=True)
            click.secho()

    def clean_cache(deploy_conf):
        click.secho("开始清理执行缓存", blink=True, bold=True)

        project_name = deploy_conf['metadata']['name']

        if "layer" in deploy_conf.keys():
            if os.path.exists(deploy_conf["layer"]["package_name"]):
                os.remove(deploy_conf["layer"]["package_name"])

            def keep_num():
                phlayer = ph_layer.PhLayer()
                resp = phlayer.get({'name': project_name})
                versions = resp['LayerVersions'] if resp else []
                versions = [version['Version'] for version in versions]

                if len(versions) > dv.LAMBDA_LAYER_MAX_VERSION_NUM:
                    for version in versions[dv.LAMBDA_LAYER_MAX_VERSION_NUM:]:
                        phlayer.delete({'name': project_name, 'version': version})

            keep_num()

        if "lambda" in deploy_conf.keys():
            if os.path.exists(deploy_conf["lambda"]["package_name"]):
                os.remove(deploy_conf["lambda"]["package_name"])

            def keep_num():
                phlambda = ph_lambda.PhLambda()

                resp = phlambda.get({'name': project_name})
                versions = resp['Versions'] if resp else []
                versions = [version['Version'] for version in versions]

                if len(versions) > dv.LAMBDA_FUNCTION_MAX_VERSION_NUM:
                    versions.remove('$LATEST')
                    for version in versions[dv.LAMBDA_FUNCTION_MAX_VERSION_NUM:]:
                        phlambda.delete({'name': project_name, 'version': version})

            keep_num()

        click.secho("执行缓存清理完成", fg='green', blink=True, bold=True)
        click.secho()

    with open(dv.DEPLOY_FILE_LOCAL_NAME) as f:
        all_conf = yaml.safe_load(f)

    # ensure project name
    if name == "":
        project_name = list(all_conf.keys())[0]
    elif name in all_conf.keys():
        project_name = name
    else:
        click.secho("project '%s' is not exists" % name, fg='red', blink=True, bold=True)
        return

    click.secho("开始部署 '%s'" % project_name, fg='green', blink=True, bold=True)

    deploy_conf = all_conf[project_name]
    deploy_conf['metadata']['version'] = time.strftime("%Y-%m-%d-%H%M%S", time.localtime())

    # filter operator
    all_operator = {"lib": "layer", "code": "lambda", "api": "gateway"}
    if "defalut" == oper:
        data = {"code": "", "api": ""}
    elif "all" == oper:
        data = {"lib": "", "code": "", "api": ""}
    else:
        data = {oper: ""}

    for not_oper in all_operator.keys() - set(data.keys()):
        del deploy_conf[all_operator[not_oper]]

    apply(deploy_conf)
    clean_cache(deploy_conf)

    click.secho("部署成功 '%s'" % project_name, fg='green', blink=True, bold=True)



import os
import re
import json
import subprocess

from phcli.ph_errs.ph_err import *
from .ph_rt_base import PhRTBase
from phcli.define_value import CLI_CLIENT_VERSION
from phcli.ph_max_auto import define_value as dv
from phcli.ph_max_auto.ph_config.phconfig.phconfig import PhYAMLConfig


class PhRTPython3(PhRTBase):
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    def c9_create_init(self, path=None):
        if not path:
            path = self.job_path + "/__init__.py"
        subprocess.call(["touch", path])

    def c9_create_phmain(self, path=None):
        if not path:
            path = self.job_path

        config = PhYAMLConfig(path)
        config.load_yaml()
        f_lines = self.phs3.open_object_by_lines(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_PHMAIN_FILE_PY)
        with open(path + "/phmain.py", "w") as file:
            s = []
            for arg in config.spec.containers.args:
                s.append(arg.key)

            for line in f_lines:
                line = line + "\n"
                if line == "$alfred_debug_execute\n":
                    file.write("@click.command()\n")
                    for must in dv.PRESET_MUST_ARGS.split(","):
                        file.write("@click.option('--{}')\n".format(must.strip()))
                    for arg in config.spec.containers.args:
                        file.write("@click.option('--" + arg.key + "')\n")
                    for output in config.spec.containers.outputs:
                        file.write("@click.option('--" + output.key + "')\n")
                    file.write("""def debug_execute(**kwargs):
    try:
        args = {"name": "$alfred_name"}
        outputs = [$alfred_outputs]

        args.update(kwargs)
        result = exec_before(**args)

        args.update(result if isinstance(result, dict) else {})
        result = execute(**args)

        args.update(result if isinstance(result, dict) else {})
        result = exec_after(outputs=outputs, **args)

        return result
    except Exception as e:
        logger = phs3logger(kwargs["job_id"])
        logger.error(traceback.format_exc())
        print(traceback.format_exc())
        raise e
      
@click.option('--job_args_name')
def online_debug_execute(**kwargs):
    try:
        args = {"name": "$alfred_name"}
        outputs = [$alfred_outputs]
        
        args.update(kwargs)
        result = exec_before(**args)

        args.update(result if isinstance(result, dict) else {})
        result = execute(**args)

        args.update(result if isinstance(result, dict) else {})
        result = exec_after(outputs=outputs, **args)

        return result
    except Exception as e:
        logger = phs3logger(kwargs["job_id"])
        logger.error(traceback.format_exc())
        print(traceback.format_exc())
        raise e
"""
                               .replace('$alfred_outputs', ', '.join(['"'+output.key+'"' for output in config.spec.containers.outputs])) \
                               .replace('$alfred_name', config.metadata.name)
                               )
                else:
                    file.write(line)

    def c9_create(self, **kwargs):
        # 1. /__init__.py file
        self.c9_create_init()

        # 2. /phjob.py file
        self.phs3.download(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_PHJOB_FILE_PY, self.job_path + "/phjob.py")
        with open(self.job_path + "/phjob.py", "a") as file:
            file.write("""def execute(**kwargs):\n""")

            file.write("""    logger = phs3logger(kwargs["job_id"], LOG_DEBUG_LEVEL)\n""")

            if self.command == 'submit':
                file.write('    spark = kwargs["spark"]()')

            file.write("""
    result_path_prefix = kwargs["result_path_prefix"]
    depends_path = kwargs["depends_path"]
    
    logger.info(kwargs["a"])
    logger.info(kwargs["b"])
    logger.info(kwargs["c"])
    logger.info(kwargs["d"])
    return {}
""")

        # 3. /phmain.py file
        self.c9_create_phmain()

    def jupyter_create(self, **kwargs):
        path = self.job_path + ".ipynb"
        dir_path = "/".join(path.split('/')[:-1])
        subprocess.call(['mkdir', '-p', dir_path])

        f_lines = self.phs3.open_object_by_lines(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_JUPYTER_PYTHON_FILE)
        with open(path, "w") as file:
            for line in f_lines:
                line = line.replace('$name', self.name) \
                            .replace('$runtime', self.runtime) \
                            .replace('$command', self.command) \
                            .replace('$timeout', str(self.timeout)) \
                            .replace('$user', os.getenv('USER', 'unknown')) \
                            .replace('$group', self.group) \
                            .replace('$ide', self.ide) \
                            .replace('$access_key', os.getenv('AWS_ACCESS_KEY_ID', "NULL")) \
                            .replace('$secret_key', os.getenv('AWS_SECRET_ACCESS_KEY', "NULL"))
                file.write(line)

    def create(self, **kwargs):
        if self.ide == 'c9':
            self.c9_create(**kwargs)
        elif self.ide == 'jupyter':
            self.jupyter_create(**kwargs)
        else:
            raise exception_function_not_implement

    def c9_to_jupyter(self, source_path, target_path, exts):
        # 创建json文件
        subprocess.run(["touch", target_path])

        with open(target_path, "w") as file:
            indentation = 4
            first_cell_index = 0
            second_cell_index = 1
            redundantCell_begin_index = 2
            redundantCell_end_index = 5

            # 从S3中获取jupyter模板
            template_str = self.phs3.open_object(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_JUPYTER_PYTHON_FILE)
            # 字符串转换成字典
            data = json.loads(template_str)
            # phconf.yaml中获取数据
            config = PhYAMLConfig(source_path)
            config.load_yaml()
            job_name = config.metadata.name
            job_runtime = config.spec.containers.runtime
            job_command = config.spec.containers.command
            job_timeout = str(config.spec.containers.timeout)

            ##### 处理第一个输入框 #####
            first_cell_source = data['cells'][first_cell_index]['source']
            # 写入基本信息
            for index, old_str in enumerate(first_cell_source):
                first_cell_source[index] = old_str.replace('$name', job_name) \
                    .replace('$runtime', job_runtime) \
                    .replace('$command', job_command) \
                    .replace('$timeout', job_timeout)

            # 写入input_args数据
            input_index = first_cell_source.index("a = 123\n")
            first_cell_input_before_row = first_cell_source[:input_index]
            first_cell_input_after_row = first_cell_source[input_index+2:]
            new_input_row = []
            for arg in config.spec.containers.args:
                arg_key = arg.key
                arg_value = arg.value
                if str.isdigit(str(arg_value)):
                    new_input_row.insert(input_index, arg_key + " = " + str(arg_value) + "\n")
                else:
                    new_input_row.insert(input_index, arg_key + " = '" + str(arg_value) + "'\n")
            first_cell_source = first_cell_input_before_row + new_input_row + first_cell_input_after_row

            # 写入output_args数据
            output_index = first_cell_source.index("c = 'abc'\n")
            first_cell_output_before_row = first_cell_source[:output_index]
            first_cell_output_after_row = first_cell_source[output_index+2:]
            new_output_row = []
            for output in config.spec.containers.outputs:
                output_key = output.key
                output_value = output.value
                if str.isdigit(str(output_value)):
                    new_output_row.insert(output_index, output_key + " = " + str(output_value) + "\n")
                else:
                    new_output_row.insert(output_index, output_key + " = '" + str(output_value) + "'\n")
            first_cell_source = first_cell_output_before_row + new_output_row + first_cell_output_after_row
            data['cells'][first_cell_index]['source'] = first_cell_source
            ##### 处理第一个输入框 #####


            ##### 处理第二个输入框 #####
            second_cell_source = data['cells'][second_cell_index]['source']
            for index, old_str in enumerate(second_cell_source):
                second_cell_source[index] = old_str.replace('$name', job_name) \
                                                   .replace('$runtime', job_runtime) \
                                                   .replace('$command', job_command) \
                                                   .replace('$timeout', job_timeout) \
                                                   .replace('$user', os.getenv('USER', 'unknown')) \
                                                   .replace('$group', exts['group']) \
                                                   .replace('$ide', exts['ide']) \
                                                   .replace('$access_key', os.getenv('AWS_ACCESS_KEY_ID', "NULL")) \
                                                   .replace('$secret_key', os.getenv('AWS_SECRET_ACCESS_KEY', "NULL"))
            data['cells'][second_cell_index]['source'] = second_cell_source
            ##### 处理第二个输入框 #####


            ##### 删除data中多余的cell #####
            del data['cells'][redundantCell_begin_index:redundantCell_end_index]
            ##### 删除data中多余的cell #####


            ##### phjob 的内容 copy 到 .ipynb 下的其余 source 中 #####
            with open(source_path + "/phjob.py", "r") as phjob_flie:
                line = phjob_flie.readline()
                while line:
                    while line.startswith('def') or line.startswith('@') or line.startswith('    # %%'):
                        demo = {
                            "cell_type": "code",
                            "execution_count": None,
                            "metadata": {},
                            "outputs": [],
                            "source": []
                        }
                        if line.startswith('@'):
                            demo['source'].append(line)
                            line = phjob_flie.readline()
                        demo['source'].append(line)
                        line = phjob_flie.readline()
                        while not line.startswith('def') and not line.startswith('@') and not line.startswith('    # %%') and not line == "":
                            demo['source'].append(line)
                            line = phjob_flie.readline()
                        data['cells'].append(demo)
                    line = phjob_flie.readline()

                execute_indexes = []
                for cell in data['cells']:
                    for source in cell['source']:
                        if source.startswith('def execute') or source.startswith('    # %%'):
                            execute_index = data['cells'].index(cell)
                            execute_indexes.append(execute_index)

                for execute_index in execute_indexes:
                    empty_source = []
                    for execute_source_str in data['cells'][execute_index]['source']:
                        if len(execute_source_str) - len(execute_source_str.lstrip()) >= indentation:
                            if execute_source_str.lstrip().startswith('logger = '):
                                continue
                            elif execute_source_str.lstrip().startswith('spark = '):
                                continue
                            if execute_source_str.lstrip().startswith('result_path_prefix = '):
                                continue
                            if execute_source_str.lstrip().startswith('depends_path = '):
                                continue
                            if execute_source_str.lstrip().startswith('### input args ###'):
                                continue
                            if execute_source_str.lstrip().startswith('### output args ###'):
                                continue

                            # logger 替换为 print
                            execute_source_str = re.sub(r'\t*logger\.\w*\(([\'|\"]?.*?[\'|\"]?)\)$', r"print(\1)", execute_source_str)
                            # 对简单 kwargs 删除，如 a = kwargs['a']
                            kv_check = re.sub(r'\s*(.*)\s*=\s*kwargs\[[\'|\"](\w*?)[\'|\"]\]', r"\1,\2", execute_source_str)
                            kv_check = kv_check.split(",")
                            if len(kv_check) == 2 and kv_check[0].strip() == kv_check[1].strip():
                                continue

                            # 对复杂 kwargs 替换，如 a = kwargs['b'] + c ==> a = b + c
                            execute_source_str = re.sub(r'^(.*)kwargs\[[\'|\"](\w*)[\'|\"]\](.*)', r"\1\2\3", execute_source_str)
                            empty_source.append(execute_source_str[indentation:])
                        elif execute_source_str.lstrip().startswith('#'):
                            empty_source.append(execute_source_str)
                    data['cells'][execute_index]['source'] = empty_source
            ##### phjob 的内容 copy 到 .ipynb 下的其余 source 中 #####

            # 把 data 从字典转换成 json 格式, indent=1 进行换行，ensure_ascii防止汉字转成Unicode码
            json_str = json.dumps(data, indent=1, ensure_ascii=False)
            file.write(json_str)

    def jupyter_to_c9(self, source_path, target_path, exts):
        def get_ipynb_map_by_key(source, key):
            """
            获取 ipynb 中定义的配置
            :param source: ipynb 中的文本行
            :param key: 需要获得的字典，可以是 config、input args、output args
            :return:
            """
            range = []
            for i, row in enumerate(source):
                if "== {} ==".format(key) in row:
                    range.append(i)
            source = source[range[0]+1: range[1]]

            result = {}
            for row in source:
                if row and '=' in row:
                    r = row.split('=')
                    result[r[0].strip()] = eval(r[-1].strip())
            return result

        # 1. 检查是否存在
        if not os.path.exists(source_path):
            raise exception_file_not_exist

        # 2. 创建目标文件夹
        subprocess.call(["mkdir", "-p", target_path])

        # 3. 读取源文件的配置和输入输出参数
        with open(source_path, 'r') as rf:
            ipynb_dict = json.load(rf)
            source = ipynb_dict['cells'][0]['source']
            cm = get_ipynb_map_by_key(source, 'config')
            im = get_ipynb_map_by_key(source, 'input args')
            om = get_ipynb_map_by_key(source, 'output args')

        # 4. 将读取的配置和参数写到 phconf.yaml 中
        input_str = ["- key: {}\n        value: '{}'".format(k, v) for k, v in im.items()]
        input_str = '\n      '.join(input_str)
        output_str = ["- key: {}\n        value: '{}'".format(k, v) for k, v in om.items()]
        output_str = '\n      '.join(output_str)
        self.create_phconf_file(path=target_path, input_str=input_str, output_str=output_str,
                                name=cm['job_name'], runtime=cm['job_runtime'],
                                command=cm['job_command'], timeout=cm['job_timeout'])

        # python 需要 __init__.py 文件
        if cm['job_runtime'] == 'python3':
            # 5. 创建 /__init__.py file
            self.c9_create_init(target_path + "/__init__.py")

        # 6. 生成 /phmain.* file
        self.c9_create_phmain(target_path)

        # 8. phconf 转为 args.properties
        self.yaml2args(target_path)

        self.phs3.download(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_PHJOB_FILE_PY, target_path + "/phjob.py")
        with open(target_path + "/phjob.py", "a") as file:
            file.write("""def execute(**kwargs):
    logger = phs3logger(kwargs["job_id"], LOG_DEBUG_LEVEL)
    spark = kwargs['spark']()
    result_path_prefix = kwargs["result_path_prefix"]
    depends_path = kwargs["depends_path"]
    
    ### input args ###
""")
            # 取入参
            for input in im:
                file.write("    {key} = kwargs['{key}']\n".format(key=input))
            file.write("    ### input args ###\n")
            file.write("    \n")

            # 取出参
            file.write("    ### output args ###\n")
            for output in om:
                file.write("    {key} = kwargs['{key}']\n".format(key=output))
            file.write("    ### output args ###\n")
            file.write("\n")

            # copy 逻辑代码
            for row in ipynb_dict['cells'][2]['source']:
                row = re.sub(r'(^\s*)print(\(.*)', r"\1logger.debug\2", row)
                row = re.sub(r'(^.*)kwargs\[[\\"|\'](.*)[\\"|\']\](.*)', r"\1\2\3", row)
                file.write('    ' + row)
            for cell in ipynb_dict['cells'][3:]:
                if cell['source'][0].startswith('# %%'):
                    for row in cell['source']:
                        row = re.sub(r'(^\s*)print(\(.*)', r"\1logger.debug\2", row)
                        row = re.sub(r'(^.*)kwargs\[[\\"|\'](.*)[\\"|\']\](.*)', r"\1\2\3", row)
                        file.write('    ' + row)
                else:
                    for row in cell['source']:
                        row = re.sub(r'(^\s*)print(\(.*)', r"\1logger.debug\2", row)
                        row = re.sub(r'(^.*)kwargs\[[\\"|\'](.*)[\\"|\']\](.*)', r"\1\2\3", row)
                        file.write(row)
                file.write('\n')

    def submit_run(self, **kwargs):
        submit_conf = {
            "jars": "s3a://ph-platform/2020-11-11/jobs/python/phcli/common/aws-java-sdk-bundle-1.11.828.jar,"
                    "s3a://ph-platform/2020-11-11/jobs/python/phcli/common/hadoop-aws-3.2.1.jar",
        }
        submit_file = {
            "py-files": "s3a://" + dv.TEMPLATE_BUCKET + "/" + dv.CLI_VERSION + dv.DAGS_S3_PHJOBS_PATH + "common/phcli-{}-py3.8.egg,".format(CLI_CLIENT_VERSION) +
                        self.submit_prefix + "phjob.py",
        }
        submit_main = self.submit_prefix + "phmain.py"

        super().submit_run(submit_conf=submit_conf,
                           submit_file=submit_file,
                           submit_main=submit_main)

    def script_run(self, **kwargs):
        self.phs3.download(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + self.s3_job_path + "/phmain.py", 'phmain.py')
        self.phs3.download(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + self.s3_job_path + "/phjob.py", 'phjob.py')
        entrypoint = ['python3', './phmain.py']
        super().script_run(entrypoint=entrypoint)





import os
import subprocess

from .ph_ide_base import PhIDEBase, exception_file_not_exist


class PhIDEC9(PhIDEBase):
    """
    针对 C9 环境的执行策略
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.logger.debug('maxauto PhIDEC9 init')
        self.logger.debug(self.__dict__)

    def create(self, **kwargs):
        """
        c9 的创建过程
        """
        self.logger.debug('maxauto ide=c9 的 create 实现')
        self.logger.debug(self.__dict__)

        self.check_path(self.job_path)
        subprocess.call(["mkdir", "-p", self.job_path])

        # 创建 /phconf.yaml file
        input_str = [k.strip() for k in self.inputs.split(',')]
        input_str = ["- key: " + i + "\n        value: \"abc\"" for i in input_str]
        input_str = '\n      '.join(input_str)
        output_str = [k.strip() for k in self.outputs.split(',')]
        output_str = ["- key: " + i + "\n        value: \"abc\"" for i in output_str]
        output_str = '\n      '.join(output_str)
        self.table_driver_runtime_inst(self.runtime)(**self.__dict__).create_phconf_file(self.job_path, input_str=input_str, output_str=output_str, **self.__dict__)

        super().create()

    def complete(self, **kwargs):
        """
        c9 的补全过程
        """
        self.logger.debug('maxauto ide=c9 的 complete 实现')
        self.logger.debug(self.__dict__)
        self.logger.error('maxauto --ide=c9 时，不支持 complete 子命令')
        raise Exception("maxauto --ide=c9 时，不支持 complete 子命令")

    def dag_copy_job(self, **kwargs):
        """
        maxauto dag 时 copy c9 环境下生成的 job
        """
        self.logger.debug('maxauto ide=c9 的 dag_copy_job 实现')
        self.logger.debug(self.__dict__)

        job_name = kwargs['job_name'].replace('.', '_')
        job_full_path = self.project_path + self.job_prefix + kwargs['job_name'].replace('.', '/')

        if not os.path.exists(job_full_path):
            raise exception_file_not_exist

        subprocess.call(["cp", '-r', job_full_path, self.dag_path + job_name])
        self.table_driver_runtime_inst(self.runtime)(**self.__dict__).yaml2args(self.dag_path + job_name)



import boto3

s3_bucket_name = "ph-apis"

def role_part():
    s3_file_path = "template/trust-policy.json"

    # with open ("../file/trust-policy.json", "r") as myfile:
    #     loads = json.load(myfile)
    #     print(loads)

    s3 = boto3.client('s3')
    object = s3.get_object(Bucket=s3_bucket_name, Key=s3_file_path)
    str = object["Body"].read().decode('utf-8')
    # str = json.load(object.get()["Body"])
    #
    client = boto3.client('iam')
    resp = client.create_role(RoleName='python-lambda-example-role2', AssumeRolePolicyDocument=str)
    print(resp)

    # iam = boto3.resource('iam')
    # role = iam.Role('python-lambda-example-role2')
    # print(role)
    # print(role.assume_role_policy_document)
    # policy_iterator = role.policies.all()
    # for policy in policy_iterator:
    #     print(policy)

# role_part()


def s3_part():
    s3_file_path = "template/trust-policy.json"
    s3 = boto3.resource('s3')

    # 列出所有库
    # for bucket in s3.buckets.all():
    #     print(bucket.name)

    # 上传文件
    data = open('../../file/ph_lmd/trust-policy.json', 'rb')
    s3.Bucket(s3_bucket_name).put_object(Key=s3_file_path, Body=data)
# s3_part()


def api_gateway_part():
    client = boto3.client('apigateway')

# api_gateway_part()

print("nodejs 支持")
print("项目移植")
print("进度，日志记录，回滚，接续执行")
print("保证版本一致性，可随时切换回来")
print("生成一份执行文件到s3")



# -*- coding: utf-8 -*-

CLI_VERSION = "2020-08-10"

ASSUME_ROLE_ARN = 'YXJuOmF3cy1jbjppYW06OjQ0NDYwMzgwMzkwNDpyb2xlL1BoLUNsaS1MbWQ='
ASSUME_ROLE_EXTERNAL_ID = 'Ph-Cli-Lmd'

DEFAULT_BUCKET = "ph-platform"
DEFAULT_TEMPLATE_DIR = "/template/python/phcli/lmd/"
DEFAULT_LAYER_DIR = "/layers/#runtime#/#name#/"
DEFAULT_LAMBDA_DIR = "/functions/#runtime#/#name#/"

LAMBDA_LAYER_MAX_VERSION_NUM = 5
LAMBDA_FUNCTION_MAX_VERSION_NUM = 5
LAMBDA_FUNCTION_ALIAS_NAME_CUR = 'current'
LAMBDA_FUNCTION_ALIAS_NAME_PREV = 'previous'

DEPLOY_FILE_TEMPLATE_NAME = 'ph-lambda-deploy-template.yaml'
DEPLOY_FILE_LOCAL_NAME = '.ph-lambda-deploy.yaml'



import os
import pytest
from phcli.ph_data_clean.__main__ import clean
from phcli.ph_data_clean.util.yaml_utils import load_by_dir
from phcli.ph_data_clean.model.clean_result import Tag

PROJECT_NAME = 'phdagcommand'


@pytest.mark.skip("util")
def chdir():
    if not os.getcwd().endswith(PROJECT_NAME):
        os.chdir('..')
        chdir()


chdir()


def test_all():
    test_files = 'file/ph_data_clean/s3_test_data/'
    test_datas = load_by_dir(test_files)
    for test_data in test_datas:
        result = clean(test_data)
        for res in result:
            if res.tag == Tag.SUCCESS:
                print()
                print('success  ')
            elif res.tag == Tag.WARNING:
                print()
                print('warning  ', res.data["UPDATE_LABEL"], "    ", res.data["PHA_ID"], "    ", res.err_msg)
            else:
                print()
                print(str(res))


test_all()



"""
# phcli maxauto submit 测试方法
shell> python phcli maxauto --cmd submit -p test \
-c "{'CONF__spark.pyspark.python': '/usr/bin/python3', \
'CONF__spark.driver.memory': '10g', 'OTHER__num-executors': '4'}" \
'{"a": "dag_run.conf", "b": "2"}'
"""



# -*- coding: utf-8 -*-

import sys
import click
from phcli.ph_lmd.model import ph_layer, ph_lambda, ph_gateway
from phcli.ph_errs.ph_err import PhException


@click.command('model', short_help='专项部署特定资源')
@click.option('-o', '--operation', required=True, help='执行行为',
              type=click.Choice(['package', 'create', 'lists', 'get',
                                 'update', 'apply', 'stop', 'start', 'delete']))
@click.option('-m', '--model', required=True, help='操作行为',
              type=click.Choice(['layer', 'lambda', 'gateway']))
@click.argument('argv', nargs=-1)
def model(operation, model, argv):
    """
    用于快速打包和部署 AWS Lambda 和 API Gateway

\b
操作名有如下：
    package : 对 lambda 的 layer 或者 code 打成 zip 包
    create  : 创建 lambda 的 role, layer 或者 code 或者 API Gateway 的一级资源
    lists   : 获取所有资源实例
    get     : 获取指定资源实例
    update  : 更新 lambda 的 layer 或者 code 或者 API Gateway 的一级资源
    apply   : 发布或更新 lambda 的 layer 或者 code 或者 API Gateway 的一级资源
    stop    : 使 lambda 或者 API Gateway 停止接受请求
    start   : 重新使 lambda 或者 API Gateway 接受请求
    delete  : 删除 lambda 的 layer 或者 code, 或者 API Gateway 的一级资源

\b
资源名有如下：
    layer   : lambda 的依赖层
    lambda  : lambda 的源代码
    gateway : lambda 的触发器 API Gateway
    """
    return fineness_func(operation, model, argv)


def fineness_func(operator, model, argv):
    """
    粒度功能使用
    :return:
    """

    def get_model_inst(model):
        model_switcher = {
            "layer": ph_layer.PhLayer,
            "lambda": ph_lambda.PhLambda,
            "gateway": ph_gateway.PhGateway,
        }
        return model_switcher.get(model, "Invalid model")()

    def get_oper_inst(model_inst, oper):
        if oper == "package":
            return model_inst.package
        elif oper == "create":
            return model_inst.create
        elif oper == "lists":
            return model_inst.lists
        elif oper == "get":
            return model_inst.get
        elif oper == "update":
            return model_inst.update
        elif oper == "apply":
            return model_inst.apply
        elif oper == "stop":
            return model_inst.stop
        elif oper == "start":
            return model_inst.start
        elif oper == "delete":
            return model_inst.delete
        else:
            raise PhException("Invalid operator")

    def argv2map(argv):
        for arg in argv:
            arr = arg.split('=', 1)

            if len(arr) == 1:
                yield arr[0], ''
            else:
                yield arr[0], arr[1]

    argv = dict(argv2map(argv))

    inst = get_oper_inst(get_model_inst(model), operator)

    if 'h' in argv.keys() or 'help' in argv.keys():
        click.secho(inst.__doc__, fg='green', blink=True, bold=True)
        sys.exit(2)

    click.secho(inst(argv), fg='green', blink=True, bold=True)



from phcli.ph_errs.ph_err import exception_not_found_preset_job


def preset_factory(context, **kwargs):
    if 'preset.write_asset' == kwargs['job_name']:
        from phcli.ph_max_auto.ph_preset_jobs.write_asset import copy_job
        copy_job(context, **kwargs)
    else:
        raise exception_not_found_preset_job



# -*- coding: utf-8 -*-

import json
from enum import Enum


class Tag(Enum):
    SUCCESS = 1
    EMPTY_DICT = 0
    MISSING_COL = -1
    WARNING = 2
    PH_ERR = -9999
    UNDEFINED = -888


class CleanResult(object):
    """
    清洗结果
    """

    def __init__(self, data, metadata, raw_data, tag, err_msg = ''):
        self.data = data
        self.metadata = metadata
        self.raw_data = raw_data
        self.tag = tag
        self.err_msg = err_msg

    def __str__(self):
        result = {
            "data": self.data,
            "metadata": self.metadata,
            "raw_data": self.raw_data,
            "tag": self.tag.value,
            "errMsg": self.err_msg
        }
        return json.dumps(result, ensure_ascii=False)



import click
from phcli.ph_admin import pg
from phcli.ph_admin.ph_models import Scope, Role


@click.group("role", short_help='角色管理工具')
def main():
    pass


@click.command("create", short_help='创建角色')
@click.option("-n", "--name", help="角色名", prompt="角色名")
@click.option("-d", "--description", help="角色描述", prompt="角色描述")
@click.option("-s", "--scope", multiple=True, help="角色权限")
def create_role(**kwargs):
    scopes = []
    for scope in kwargs['scope']:
        for scope in pg.query(Scope(), name=scope):
            scopes.append(scope.id)
    kwargs['scope'] = scopes

    result = pg.insert(Role(**kwargs))
    click.secho('添加成功'+str(result), fg='green', blink=True, bold=True)
    pg.commit()


@click.command("update", short_help='更新角色')
def update_role():
    click.secho('未实现', fg='red', blink=True, bold=True)
    pass


@click.command("list", short_help='列举角色')
def list_role():
    for role in pg.query(Role()):
        click.secho(str(role), fg='green', blink=True, bold=True)


@click.command("get", short_help='查找角色')
@click.option("-n", "--name", help="角色名", default=None)
def get_role(**kwargs):
    for role in pg.query(Role(**kwargs)):
        click.secho(str(role), fg='green', blink=True, bold=True)


@click.command("delete", short_help='删除角色')
@click.option("-n", "--name", help="角色名", prompt="角色名")
def delete_role(**kwargs):
    for role in pg.delete(Role(**kwargs)):
        click.secho(str(role), fg='green', blink=True, bold=True)
    pg.commit()


main.add_command(create_role)
main.add_command(update_role)
main.add_command(list_role)
main.add_command(get_role)
main.add_command(delete_role)



import os
import pytest
from phcli.ph_lmd.runtime.python_rt import PythonRT


python_rt = PythonRT()


@pytest.mark.skip("Pipenv is slow for the first time")
def test_python_rt_pkg_layer_pipenv():
    args = {
        "package_name": "test_python_rt_layer_pipenv.zip",
        "is_pipenv": True,
    }

    python_rt.pkg_layer(args)
    assert os.path.exists(args["package_name"])
    os.remove(args["package_name"])


def test_python_rt_pkg_layer_libpath():
    args = {
        "lib_path": ".venv/lib/python3.8/site-packages/",
        "package_name": "test_python_rt_layer_libpath.zip"
    }

    python_rt.pkg_layer(args)
    assert os.path.exists(args["package_name"])
    os.remove(args["package_name"])


def test_python_rt_pkg_code():
    args = {
        "code_path": "./phlmd",
        "package_name": "test_python_rt_pkg_code.zip"
    }
    package_name = "test_python_rt_pkg_code.zip"
    python_rt.pkg_code(args)
    assert os.path.exists(package_name)
    os.remove(package_name)



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the usage of class phCommand,
which help users to create, update, and publish the jobs they created.
"""
import os
import click
import datetime
from phcli.ph_max_auto import define_value as dv
from phcli.ph_max_auto.ph_metric.ph_metric import PhMetric
from phcli.ph_max_auto.phcontext.phcontextfacade import PhContextFacade


def put_metric(api, call_state):
    context_args["metric"].put_call_cli_count(
        api,
        dag_name=context_args.get('group', ''),
        job_name=context_args.get('name', ''),
        call_state=call_state
    )
    context_args["metric"].put_call_cli_duration_time(
        api,
        dag_name=context_args.get('group', ''),
        job_name=context_args.get('name', ''),
        call_state=call_state,
        duration_time=(datetime.datetime.now()-context_args["starttime"]).microseconds
    )


context_args = {}


@click.group("maxauto")
@click.option("-I", "--ide",
              help="You IDE.",
              type=click.Choice(["c9", "jupyter"]),
              default=os.getenv(dv.ENV_CUR_IDE_KEY, dv.ENV_CUR_IDE_DEFAULT))
@click.option("-R", "--runtime",
              help="You use programming language.",
              type=click.Choice(["python3", "r"]),
              default=os.getenv(dv.ENV_CUR_RUNTIME_KEY, dv.ENV_CUR_RUNTIME_DEFAULT))
@click.option("--log_level",
              help="You log print level.",
              type=click.Choice(["debug", "info", "warn", 'error']),
              default="warn")
def maxauto(**kwargs):
    """
    The Pharbers Max Job Command Line Interface (CLI)
    """
    global context_args
    context_args = {k: str(v).strip() for k, v in kwargs.items()}
    context_args["metric"] = PhMetric()
    context_args["starttime"] = datetime.datetime.now()


@maxauto.command("create")
@click.option("-g", "--group",
              prompt="The job group is",
              help="The job group.",
              default="")
@click.option("-n", "--name",
              prompt="The job name is",
              help="The job name.")
@click.option("-C", "--command",
              prompt="The job command is",
              help="The job command.",
              type=click.Choice(["submit", "script"]),
              default="submit")
@click.option("-T", "--timeout",
              prompt="The job timeout (min) is",
              help="The job timeout (min) .",
              type=click.FLOAT,
              default="10")
@click.option("-i", "--inputs",
              prompt="The job inputs is",
              help="The job inputs.",
              default="a,b")
@click.option("-o", "--outputs",
              prompt="The job outputs is",
              help="The job outputs.",
              default="c,d")
def create(**kwargs):
    """
    创建一个 Job
    """
    try:
        context_args.update({k: str(v).strip() for k, v in kwargs.items()})
        PhContextFacade(**context_args).command_create_exec()
    except Exception as e:
        put_metric("maxauto.create", "failed")
        click.secho("创建失败: " + str(e), fg='red', blink=True, bold=True)
    else:
        put_metric("maxauto.create", "success")
        click.secho("创建完成", fg='green', blink=True, bold=True)


@maxauto.command("complete")
@click.option("-g", "--group",
              prompt="The job group is, [ALL] is all job complete",
              help="The job group.",
              default="")
@click.option("-n", "--name",
              prompt="The job name is, [ALL] is all job complete in group",
              help="The job name.",
              default="ALL")
@click.option("-s", "--strategy",
              prompt="The job strategy complete is [s2c = special to common, c2s = common to special]",
              help="The job strategy complete. [s2c = special to common, c2s = common to special]",
              type=click.Choice(["s2c", "c2s", 'auto']),
              default="s2c")
def complete(**kwargs):
    """
    补全一个 Job
    """
    try:
        context_args.update({k: str(v).strip() for k, v in kwargs.items()})
        PhContextFacade(**context_args).command_complete_exec()
    except Exception as e:
        raise e
        put_metric("maxauto.complete", "failed")
        click.secho("补全失败: " + str(e), fg='red', blink=True, bold=True)
    else:
        put_metric("maxauto.complete", "success")
        click.secho("补全完成", fg='green', blink=True, bold=True)


@maxauto.command("run")
@click.option("-g", "--group",
              prompt="The job group is",
              help="The job group.",
              default="")
@click.option("-n", "--name",
              prompt="The job name is",
              help="The job name.")
def run(**kwargs):
    """
    运行一个 Job
    """
    try:
        context_args.update({k: str(v).strip() for k, v in kwargs.items()})
        PhContextFacade(**context_args).command_run_exec()
    except Exception as e:
        put_metric("maxauto.run", "failed")
        click.secho("运行失败: " + str(e), fg='red', blink=True, bold=True)
    else:
        put_metric("maxauto.run", "success")
        click.secho("运行完成", fg='green', blink=True, bold=True)


@maxauto.command("combine")
@click.option("-n", "--name",
              prompt="The dag name is",
              help="The dag name.")
@click.option("-o", "--owner",
              prompt="The dag owner is",
              help="The dag owner.",
              default="default")
@click.option("-t", "--tag",
              prompt="The dag tag is",
              help="The dag tag.",
              default="default")
@click.option("-T", "--timeout",
              prompt="The dag timeout (min) is, default=$(jobs timeout total)",
              help="The dag timeout.",
              default="")
@click.option("-j", "--jobs",
              prompt="The dag jobs is",
              help="The dag jobs.",
              default="group.test1, group.test2")
def combine(**kwargs):
    """
    关联一组 Job
    """
    try:
        context_args.update({k: str(v).strip() for k, v in kwargs.items()})
        PhContextFacade(**context_args).command_combine_exec()
    except Exception as e:
        put_metric("maxauto.combine", "failed")
        click.secho("关联失败: " + str(e), fg='red', blink=True, bold=True)
    else:
        put_metric("maxauto.combine", "success")
        click.secho("关联完成", fg='green', blink=True, bold=True)


@maxauto.command("dag")
@click.option("-n", "--name",
              prompt="The dag name is",
              help="The dag name.")
def dag(**kwargs):
    """
    通过 combine 生成一组 DAG 运行文件
    """
    try:
        context_args.update({k: str(v).strip() for k, v in kwargs.items()})
        PhContextFacade(**context_args).command_dag_exec()
    except Exception as e:
        put_metric("maxauto.dag", "failed")
        click.secho("DAG失败: " + str(e), fg='red', blink=True, bold=True)
    else:
        put_metric("maxauto.dag", "success")
        click.secho("DAG完成", fg='green', blink=True, bold=True)


@maxauto.command("publish")
@click.option("-n", "--name",
              prompt="The dag name is",
              help="The dag name.")
@click.option("-s", "--strategy",
              prompt="The publish strategy is [v2 = publish to airflow, v3 = publish tp step_functions]",
              help="The publish strategy is [v2 = publish to airflow, v3 = publish tp step_functions]",
              type=click.Choice(["v2", "v3"]),
              default="v2")
@click.option("-c", "--cluster_id",
              prompt="The emr cluster_id is",
              help="The emr cluster_id.")
@click.option("-a", "--job_args",
              prompt="The job_args is",
              help="The job_args.")
def publish(**kwargs):
    """
    发布 DAG 运行文件和相关依赖
    """
    try:
        context_args.update({k: str(v).strip() for k, v in kwargs.items()})
        PhContextFacade(**context_args).command_publish_exec()
    except Exception as e:
        print()
        put_metric("maxauto.publish", "failed")
        click.secho("发布失败: " + str(e), fg='red', blink=True, bold=True)
    else:
        put_metric("maxauto.publish", "success")
        click.secho("发布完成", fg='green', blink=True, bold=True)


@maxauto.command("recall")
@click.option("-n", "--name",
              prompt="The dag name is",
              help="The dag name.")
def recall(**kwargs):
    """
    召回 DAG 运行文件和相关依赖
    """
    try:
        context_args.update({k: str(v).strip() for k, v in kwargs.items()})
        PhContextFacade(**context_args).command_recall_exec()
    except Exception as e:
        put_metric("maxauto.recall", "failed")
        click.secho("召回失败: " + str(e), fg='red', blink=True, bold=True)
    else:
        put_metric("maxauto.recall", "success")
        click.secho("召回完成", fg='green', blink=True, bold=True)


@maxauto.command("online_run")
@click.option("--dag_name",
              prompt="The dag name is",
              help="The dag name.",
              default="")
@click.option("--job_full_name",
              prompt="The dag job_full_name is",
              help="The dag job_full_name.")
@click.option("--owner", default="owner")
@click.option("--run_id", default="run_id")
@click.option("--job_id", default="job_id")
@click.option("-c", "--context", help="online_run context", default="{}")
@click.argument('args', nargs=1, default="{}")
def online_run(**kwargs):
    """
    通过指定 Job name 在线上执行
    """
    try:
        context_args.update({k: str(v).strip() for k, v in kwargs.items()})
        PhContextFacade(**context_args).command_online_run_exec()
    except Exception as e:
        put_metric("maxauto.online_run", "failed")
        click.secho("online_run失败: " + str(e), fg='red', blink=True, bold=True)
        raise e
    else:
        put_metric("maxauto.online_run", "success")
        click.secho("online_run完成", fg='green', blink=True, bold=True)


@maxauto.command("status")
def status(**kwargs):
    """
    获取执行状态（暂无）
    """
    try:
        context_args.update({k: str(v).strip() for k, v in kwargs.items()})
        PhContextFacade(**context_args).command_status_exec()
    except Exception as e:
        put_metric("maxauto.status", "failed")
        click.secho("查看状态失败: " + str(e), fg='red', blink=True, bold=True)
    else:
        put_metric("maxauto.status", "success")
        click.secho("查看状态完成", fg='green', blink=True, bold=True)



import base64

from phcli.ph_max_auto.phcontext.ph_runtime.ph_rt_python3 import PhRTPython3
from phcli.ph_max_auto import define_value as dv
from phcli.ph_aws.ph_s3 import PhS3
from phcli.ph_aws.ph_sts import PhSts


def main():
    print("test_main方法被调用")
    phsts = PhSts().assume_role(
        base64.b64decode(dv.ASSUME_ROLE_ARN).decode(),
        dv.ASSUME_ROLE_EXTERNAL_ID,
    )
    phs3_test = PhS3(phsts=phsts)

    source_path = "./phjobs/a/b"
    target_path = "./phjobs/a/b.ipynb"
    t = PhRTPython3(phs3=phs3_test, group="a", name="b", ide="jupyter")
    t.c9_to_jupyter(source_path, target_path)


if __name__ == '__main__':
    main()
    print("success")



import click
import hashlib
from datetime import datetime
from phcli.ph_admin.ph_models import Account, Role, Partner


@click.group("user", short_help='用户管理工具')
def main():
    pass


@click.command("create", short_help='创建用户')
@click.option("-n", "--name", help="用户名", prompt="用户名")
@click.option("-p", "--password", help="用户密码", prompt="用户密码", hide_input=True, confirmation_prompt=True)
@click.option("--phoneNumber", help="用户电话", prompt="用户电话")
@click.option("-r", "--defaultRole", help="默认角色", prompt="默认角色", default='test')
@click.option("-e", "--email", help="用户邮箱", prompt="用户邮箱")
@click.option("-E", "--employer", help="所属公司", prompt="所属公司", default='test')
@click.option("--firstName", help="名", prompt="名")
@click.option("--lastName", help="姓", prompt="姓")
def create_user(**kwargs):
    from phcli.ph_admin import pg
    kwargs['phoneNumber'] = kwargs.pop('phonenumber')
    kwargs['defaultRole'] = kwargs.pop('defaultrole')
    kwargs['firstName'] = kwargs.pop('firstname')
    kwargs['lastName'] = kwargs.pop('lastname')

    sha256 = hashlib.sha256()
    sha256.update(kwargs['password'].encode('utf-8'))
    kwargs['password'] = sha256.hexdigest()
    default_role = kwargs.pop('defaultRole')
    employer = kwargs.pop('employer')

    roles = pg.query(Role(), name=default_role)
    role = roles[0] if roles else None
    if role:
        kwargs['defaultRole'] = role.id

    employers = pg.query(Partner(), name=employer)
    employer = employers[0] if employers else None
    if employer:
        kwargs['employer'] = employer.id

    account = pg.insert(Account(**kwargs))
    if role:
        role.accountRole.append(str(account.id))
        role.modified = datetime.now()
        pg.update(role)

    if employer:
        employer.employee.append(str(account.id))
        employer.modified = datetime.now()
        pg.update(employer)

    click.secho('添加成功'+str(account), fg='green', blink=True, bold=True)
    pg.commit()


@click.command("update", short_help='更新用户')
@click.option("-n", "--name", help="用户名", prompt="用户名")
@click.option("-p", "--password", help="用户密码", default=None, hide_input=True, confirmation_prompt=True)
@click.option("--phoneNumber", help="用户电话", default=None)
@click.option("-R", "--defaultRole", help="默认角色", default=None)
@click.option("-e", "--email", help="用户邮箱", default=None)
@click.option("-E", "--employer", help="所属公司", default=None)
@click.option("--firstName", help="名", default=None)
@click.option("--lastName", help="姓", default=None)
def update_user(**kwargs):
    from phcli.ph_admin import pg
    kwargs['phoneNumber'] = kwargs.pop('phonenumber')
    kwargs['defaultRole'] = kwargs.pop('defaultrole')
    kwargs['firstName'] = kwargs.pop('firstname')
    kwargs['lastName'] = kwargs.pop('lastname')

    if kwargs['password']:
        sha256 = hashlib.sha256()
        sha256.update(kwargs['password'].encode('utf-8'))
        kwargs['password'] = sha256.hexdigest()

    user = pg.query(Account(), name=kwargs['name'])
    user = user[0] if user else None
    if not user:
        click.secho('用户不存在: '+str(kwargs['name']), fg='red', blink=True, bold=True)
        return

    default_role = kwargs.pop('defaultRole')
    if default_role:
        new_role = pg.query(Role(), name=default_role)
        new_role = new_role[0] if new_role else None
        if not new_role:
            click.secho('角色不存在: '+default_role, fg='yellow', blink=True, bold=True)

        if user.defaultRole:
            old_role = pg.query(Role(), id=user.defaultRole)
            old_role = old_role[0] if old_role else None
        else:
            old_role = None

        # 修改的名字和原有的一样，判定为没变化
        if new_role and old_role and new_role.id == old_role.id:
            click.secho('角色无变化, name='+new_role.name, fg='yellow', blink=True, bold=True)
        else:
            if old_role:
                old_role.accountRole.remove(user.id)
                old_role.modified = datetime.now()
                pg.update(old_role)
                user.defaultRole = None

            if new_role:
                new_role.accountRole.append(user.id)
                new_role.modified = datetime.now()
                pg.update(new_role)
                user.defaultRole = new_role.id

    employer = kwargs.pop('employer')
    if employer:
        new_employer = pg.query(Partner(), name=employer)
        new_employer = new_employer[0] if new_employer else None
        if not new_employer:
            click.secho('公司不存在: '+employer, fg='yellow', blink=True, bold=True)

        if user.employer:
            old_employer = pg.query(Partner(), id=user.employer)
            old_employer = old_employer[0] if old_employer else None
        else:
            old_employer = None

        # 修改的名字和原有的一样，判定为没变化
        if new_employer and old_employer and new_employer.id == old_employer.id:
            click.secho('公司无变化, name='+new_employer.name, fg='yellow', blink=True, bold=True)
        else:
            if old_employer:
                old_employer.employee.remove(user.id)
                old_employer.modified = datetime.now()
                pg.update(old_employer)
                user.employer = None

            if new_employer:
                new_employer.employee.append(user.id)
                new_employer.modified = datetime.now()
                pg.update(new_employer)
                user.employer = new_employer.id

    user = pg.update(user)
    click.secho('更新成功'+str(user), fg='green', blink=True, bold=True)
    pg.commit()


@click.command("list", short_help='列举用户')
def list_user():
    from phcli.ph_admin import pg
    for a in pg.query(Account()):
        click.secho(str(a), fg='green', blink=True, bold=True)


@click.command("get", short_help='查找用户')
@click.option("-n", "--name", help="用户名", default=None)
@click.option("--phoneNumber", help="用户电话", default=None)
@click.option("-e", "--email", help="用户邮箱", default=None)
@click.option("--firstName", help="名", default=None)
@click.option("--lastName", help="姓", default=None)
def get_user(**kwargs):
    from phcli.ph_admin import pg
    for a in pg.query(Account(**kwargs)):
        click.secho(str(a), fg='green', blink=True, bold=True)


@click.command("delete", short_help='删除用户')
@click.option("-n", "--name", help="用户名", prompt="用户名")
def delete_user(**kwargs):
    from phcli.ph_admin import pg
    for user in pg.delete(Account(**kwargs)):
        if user.defaultRole:
            old_role = pg.query(Role(), id=user.defaultRole)
            old_role = old_role[0] if old_role else None
            if old_role:
                old_role.accountRole.remove(user.id)
                old_role.modified = datetime.now()
                pg.update(old_role)

        if user.employer:
            old_employer = pg.query(Partner(), id=user.employer)
            old_employer = old_employer[0] if old_employer else None
            if old_employer:
                old_employer.employee.remove(user.id)
                old_employer.modified = datetime.now()
                pg.update(old_employer)

        click.secho('删除成功'+str(user), fg='green', blink=True, bold=True)
    pg.commit()



main.add_command(create_user)
main.add_command(update_user)
main.add_command(list_user)
main.add_command(get_user)
main.add_command(delete_user)



import boto3

def get_args_from_ssm(kwargs):
    ssm_client = boto3.client('ssm', region_name='cn-northwest-1')
    response = ssm_client.get_parameter(
        Name='spark_args'
    )
    args = eval(response['Parameter']['Value'])
    return args

def delete_args_from_ssm(kwargs):
    ssm_client = boto3.client('ssm', region_name='cn-northwest-1')
    ssm_client.delete_parameter(
        Name=kwargs.get('job_args_name')
    )

if __name__ == '__main__':
    asd = get_args_from_ssm({1:1})
    print(asd)


# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This is job template for Pharbers Max Job
"""

from phcli.ph_logs.ph_logs import phs3logger, LOG_DEBUG_LEVEL


def execute(**kwargs):
    logger = phs3logger(kwargs["job_id"], LOG_DEBUG_LEVEL)
    spark = kwargs['spark']()
    result_path_prefix = kwargs["result_path_prefix"]
    depends_path = kwargs["depends_path"]

    ### input args ###
    g_project_name = kwargs['g_project_name']
    g_model_month_right = kwargs['g_model_month_right']
    g_month = kwargs['g_month']
    g_year = kwargs['g_year']
    g_current_month = kwargs['g_current_month']
    g_if_add_data = kwargs['g_if_add_data']
    depend_job_names_keys = kwargs['depend_job_names_keys']
    g_monthly_update = kwargs['g_monthly_update']
    dag_name = kwargs['dag_name']
    run_id = kwargs['run_id']
    max_path = kwargs['max_path']
    ### input args ###

    ### output args ###
    g_adding_data = kwargs['g_adding_data']
    g_raw_data_adding_final = kwargs['g_raw_data_adding_final']
    ### output args ###

    import pandas as pd
    import os
    from pyspark.sql.types import StringType, IntegerType, DoubleType, StructType, StructField
    from pyspark.sql import functions as func
    from pyspark.sql.functions import pandas_udf, PandasUDFType, udf, col
    # %%
    # 测试输入

    # g_project_name = '贝达'
    # g_month = "12"
    # g_year = "2020"
    # g_model_month_right = '201912'
    # g_current_month = '12'
    # result_path_prefix=get_result_path({"name":job_name, "dag_name":dag_name, "run_id":run_id})
    # depends_path=get_depends_path({"name":job_name, "dag_name":dag_name,
    #                                  "run_id":run_id, "depend_job_names_keys":depend_job_names_keys })

    # %%
    logger.debug('数据执行-start：补数-月更新')
    # 是否运行此job
    if g_monthly_update == "False":
        return

    if g_if_add_data != "False" and g_if_add_data != "True":
        logger.error('wrong input: g_if_add_data, False or True')
        raise ValueError('wrong input: g_if_add_data, False or True')

    # 输入
    g_model_month_right = int(g_model_month_right)
    p_product_mapping = depends_path['deal_poi_out']
    p_growth_rate = depends_path['growth_rate']
    p_price = depends_path['price']
    p_price_city = depends_path['price_city']

    # 测试输入
    g_current_month = int(g_current_month)
    p_cpa_pha_mapping = max_path + "/" + g_project_name + "/cpa_pha_mapping"

    # 月更新相关参数
    g_month = int(g_month)
    g_year = int(g_year)

    p_not_arrived = max_path + "/Common_files/Not_arrived" + str(g_year * 100 + g_current_month) + ".csv"

    # 输出
    p_adding_data = result_path_prefix + g_adding_data
    p_raw_data_adding_final = result_path_prefix + g_raw_data_adding_final

    # %%
    # =========== 数据准备，测试用 =============
    def dealIDlength(df):
        # ID不足7位的补足0到6位
        # 国药诚信医院编码长度是7位数字，cpa医院编码是6位数字。
        df = df.withColumn("ID", df["ID"].cast(StringType()))
        # 去掉末尾的.0
        df = df.withColumn("ID", func.regexp_replace("ID", "\\.0", ""))
        df = df.withColumn("ID", func.when(func.length(df.ID) < 7, func.lpad(df.ID, 6, "0")).otherwise(df.ID))
        return df

    df_cpa_pha_mapping = spark.read.parquet(p_cpa_pha_mapping)
    df_cpa_pha_mapping = df_cpa_pha_mapping.withColumnRenamed('推荐版本', 'COMMEND')
    df_cpa_pha_mapping = df_cpa_pha_mapping.select('COMMEND', 'ID', 'PHA')
    df_cpa_pha_mapping = dealIDlength(df_cpa_pha_mapping)

    df_not_arrived = spark.read.csv(p_not_arrived, header=True)
    df_not_arrived = df_not_arrived.withColumnRenamed('Date', 'DATE')

    # %%
    # =========== 数据准备 =============
    def unpivot(df, keys):
        # 功能：数据宽变长
        # 参数说明 df:dataframe,  keys 待转换表中需要保留的主键key，以list[]类型传入
        # 转换是为了避免字段类不匹配，统一将数据转换为string类型，如果保证数据类型完全一致，可以省略该句
        df = df.select(*[col(_).astype("string") for _ in df.columns])
        cols = [_ for _ in df.columns if _ not in keys]
        stack_str = ','.join(map(lambda x: "'%s', `%s`" % (x, x), cols))
        # feature, value 转换后的列名，可自定义
        df = df.selectExpr(*keys, "stack(%s, %s) as (feature, value)" % (len(cols), stack_str))
        return df

    # 一. 生成 df_original_range_raw（样本中已到的Year、Month、PHA的搭配）
    # 2017到当前年的全量出版医院
    Published_years = list(range(2017, g_year + 1, 1))
    for index, eachyear in enumerate(Published_years):
        allmonth = [str(eachyear * 100 + i) for i in list(range(1, 13, 1))]
        published_path = max_path + "/Common_files/Published" + str(eachyear) + ".csv"
        published = spark.read.csv(published_path, header=True)
        published = published.where(col('Source') == 'CPA').select('ID').distinct()
        published = dealIDlength(published)
        for i in allmonth:
            published = published.withColumn(i, func.lit(1))
        if index == 0:
            published_full = published
        else:
            published_full = published_full.join(published, on='ID', how='full')

    df_published_all = unpivot(published_full, ['ID'])
    df_published_all = df_published_all.where(col('value') == 1).withColumnRenamed('feature', 'Date') \
        .drop('value')

    # 模型前之前的未到名单（跑模型年的时候，不去除未到名单）
    # 1.当前年的未到名单
    not_arrived_current = spark.read.csv(p_not_arrived, header=True)
    not_arrived_current = not_arrived_current.select('ID', 'Date').distinct()
    not_arrived_current = dealIDlength(not_arrived_current)
    # 2.其他模型年之后的未到名单
    model_year = g_model_month_right // 100
    not_arrived_others_years = set((range(model_year + 1, g_year + 1, 1))) - set([g_year])
    if not_arrived_others_years:
        for index, eachyear in enumerate(not_arrived_others_years):
            not_arrived_others_path = max_path + "/Common_files/Not_arrived" + str(eachyear) + "12.csv"
            logger.debug(not_arrived_others_path)
            not_arrived = spark.read.csv(not_arrived_others_path, header=True)
            not_arrived = not_arrived.select('ID', 'Date').distinct()
            not_arrived = dealIDlength(not_arrived)
            if index == 0:
                not_arrived_others = not_arrived
            else:
                not_arrived_others = not_arrived_others.union(not_arrived)
        not_arrived_all = not_arrived_current.union(not_arrived_others)
    else:
        not_arrived_all = not_arrived_current

    # raw_data中每个年月的非CPA医院列表
    # df_raw_data = spark.read.parquet(p_product_mapping)
    # struct_type_product_mapping = StructType([ StructField('MIN', StringType(), True),
    #                                             StructField('PHA', StringType(), True),
    #                                             StructField('ID', StringType(), True),
    #                                             StructField('YEAR_MONTH', IntegerType(), True),
    #                                             StructField('RAW_HOSP_NAME', StringType(), True),
    #                                             StructField('BRAND', StringType(), True),
    #                                             StructField('FORM', StringType(), True),
    #                                             StructField('SPECIFICATIONS', StringType(), True),
    #                                             StructField('PACK_NUMBER', StringType(), True),
    #                                             StructField('MANUFACTURER', StringType(), True),
    #                                             StructField('MOLECULE', StringType(), True),
    #                                             StructField('SOURCE', StringType(), True),
    #                                             StructField('CORP', StringType(), True),
    #                                             StructField('ROUTE', StringType(), True),
    #                                             StructField('ORG_MEASURE', StringType(), True),
    #                                             StructField('SALES', DoubleType(), True),
    #                                             StructField('UNITS', DoubleType(), True),
    #                                             StructField('UNITS_BOX', DoubleType(), True),
    #                                             StructField('PATH', StringType(), True),
    #                                             StructField('SHEET', StringType(), True),
    #                                             StructField('CITY', StringType(), True),
    #                                             StructField('PROVINCE', StringType(), True),
    #                                             StructField('CITY_TIER', DoubleType(), True),
    #                                             StructField('MONTH', IntegerType(), True),
    #                                             StructField('YEAR', IntegerType(), True),
    #                                             StructField('MIN_STD', StringType(), True),
    #                                             StructField('MOLECULE_STD', StringType(), True),
    #                                             StructField('ROUTE_STD', StringType(), True),
    #                                             StructField('BRAND_STD', StringType(), True) ])
    # df_raw_data = spark.read.format("parquet").load(p_product_mapping, schema=struct_type_product_mapping)

    # df_original_range_raw_noncpa = df_raw_data.where(col('Source') != 'CPA').select('ID', 'YEAR_MONTH').distinct() \
    #                                     .withColumnRenamed('YEAR_MONTH', 'Date')

    # df_raw_data = spark.read.parquet(p_product_mapping)

    #################################################### 新的表里没有 Source这一列了

    struct_type = StructType([StructField('PHA', StringType(), True),
                              StructField('ID', StringType(), True),
                              StructField('PACK_ID', StringType(), True),
                              StructField('MANUFACTURER_STD', StringType(), True),
                              StructField('YEAR_MONTH', IntegerType(), True),
                              StructField('MOLECULE_STD', StringType(), True),
                              StructField('BRAND_STD', StringType(), True),
                              StructField('PACK_NUMBER_STD', IntegerType(), True),
                              StructField('FORM_STD', StringType(), True),
                              StructField('SPECIFICATIONS_STD', StringType(), True),
                              StructField('SALES', DoubleType(), True),
                              StructField('UNITS', DoubleType(), True),
                              StructField('CITY', StringType(), True),
                              StructField('PROVINCE', StringType(), True),
                              StructField('CITY_TIER', DoubleType(), True),
                              StructField('MONTH', IntegerType(), True),
                              StructField('YEAR', IntegerType(), True),
                              StructField('MOLECULE_STD_FOR_GR', StringType(), True)])
    df_raw_data = spark.read.format("parquet").load(p_product_mapping, schema=struct_type)
    df_raw_data = df_raw_data.withColumn("MIN_STD", func.format_string("%s|%s|%s|%s|%s", "BRAND_STD", "FORM_STD",
                                                                       "SPECIFICATIONS_STD", "PACK_NUMBER_STD",
                                                                       "MANUFACTURER_STD"))

    df_original_range_raw_noncpa = df_raw_data.select('ID', 'YEAR_MONTH').distinct() \
        .withColumnRenamed('YEAR_MONTH', 'DATE')
    ###################################################

    # 出版医院 减去 未到名单(月更)
    df_original_range_raw = df_published_all.join(not_arrived_all, on=['ID', 'Date'], how='left_anti')

    # 与 非CPA医院 合并
    df_original_range_raw = df_original_range_raw.union(
        df_original_range_raw_noncpa.select(df_original_range_raw.columns))

    # 匹配 PHA
    df_cpa_pha_mapping = df_cpa_pha_mapping.where(col("COMMEND") == 1) \
        .select("ID", "PHA").distinct()

    df_original_range_raw = df_original_range_raw.join(df_cpa_pha_mapping, on='ID', how='left')
    df_original_range_raw = df_original_range_raw.where(~col('PHA').isNull()) \
        .withColumn('YEAR', func.substring(col('Date'), 0, 4)) \
        .withColumn('MONTH', func.substring(col('Date'), 5, 2).cast(IntegerType())) \
        .select('PHA', 'YEAR', 'MONTH').distinct()

    # %%
    # =========== 数据执行 =============
    logger.debug('数据执行-start')
    # 1.数据准备
    # df_raw_data = spark.read.parquet(p_product_mapping)

    ## 读取 price
    # df_price = spark.read.parquet(p_price)
    struct_type_price = StructType([StructField('MIN_STD', StringType(), True),
                                    StructField('YEAR_MONTH', IntegerType(), True),
                                    StructField('CITY_TIER', DoubleType(), True),
                                    StructField('PRICE', DoubleType(), True)])
    df_price = spark.read.format("parquet").load(p_price, schema=struct_type_price)
    df_price = df_price.withColumnRenamed('PRICE', 'PRICE_TIER')

    ## 读取 growth_rate
    df_growth_rate = spark.read.parquet(p_growth_rate)
    df_growth_rate.persist()

    ## 读取 price_city
    # df_price_city = spark.read.parquet(p_price_city)
    struct_type_price_city = StructType([StructField('MIN_STD', StringType(), True),
                                         StructField('YEAR_MONTH', IntegerType(), True),
                                         StructField('CITY', StringType(), True),
                                         StructField('PROVINCE', StringType(), True),
                                         StructField('PRICE', DoubleType(), True)])
    df_price_city = spark.read.format("parquet").load(p_price_city, schema=struct_type_price_city)
    df_price_city = df_price_city.withColumnRenamed('PRICE', 'PRICE_CITY')

    # %%
    # 补数函数
    def addDate(df_raw_data, df_growth_rate):
        # 1. 原始数据格式整理， 用于补数
        df_growth_rate = df_growth_rate.select(["CITYGROUP", "MOLECULE_STD_FOR_GR"] +
                                               [name for name in df_growth_rate.columns if
                                                name.startswith("GR")]).distinct()

        df_raw_data_for_add = df_raw_data.where(col('PHA').isNotNull()) \
            .orderBy(col('YEAR').desc()) \
            .withColumnRenamed("CITY_TIER", "CITYGROUP") \
            .join(df_growth_rate, on=["MOLECULE_STD_FOR_GR", "CITYGROUP"], how="left")
        df_raw_data_for_add.persist()

        # 2. 获取所有发表医院
        # 原始数据的 PHA-Month-YEAR
        # original_range = df_raw_data_for_add.select("YEAR", "Month", "PHA").distinct()

        years = df_raw_data_for_add.select("YEAR").distinct() \
            .orderBy(df_raw_data_for_add.YEAR) \
            .toPandas()["YEAR"].values.tolist()

        df_original_range = df_original_range_raw.where(col('YEAR').isin(years))

        growth_rate_index = [i for i, name in enumerate(df_raw_data_for_add.columns) if name.startswith("GR")]

        # 3.对每年的缺失数据分别进行补数
        # 当前年：每月publish的PHA
        df_current_range_pha_month = df_original_range.where(col('YEAR') == g_year) \
            .select("MONTH", "PHA").distinct()
        # 当前年：publish的月份
        df_current_range_month = df_current_range_pha_month.select("MONTH").distinct()
        # 其他年：月份-当前年publish的月份，PHA-当前年没有publish的医院（这些医院需要补数）
        df_other_years_range = df_original_range.where(col('YEAR') != g_year) \
            .join(df_current_range_month, on="MONTH", how="inner") \
            .join(df_current_range_pha_month, on=["MONTH", "PHA"], how="left_anti")
        # 其他年与当前年的年份差值，比重计算（临近上一年比重为0.5，临近后一年比重为1）
        df_other_years_range = df_other_years_range \
            .withColumn("TIME_DIFF", (col('YEAR') - g_year)) \
            .withColumn("WEIGHT", func.when((col('YEAR') > g_year), (col('YEAR') - g_year - 0.5)).
                        otherwise(col('YEAR') * (-1) + g_year))
        # 选择比重最小的年份：用于补数的 PHA-Month-Year
        df_current_range_for_add = df_other_years_range.repartition(1).orderBy(col('WEIGHT').asc())
        df_current_range_for_add = df_current_range_for_add.groupBy("PHA", "MONTH") \
            .agg(func.first(col('YEAR')).alias("YEAR"))

        # 从 rawdata 根据 df_current_range_for_add 获取用于补数的数据
        df_current_raw_data_for_add = df_raw_data_for_add.where(col('YEAR') != g_year) \
            .join(df_current_range_for_add, on=["MONTH", "PHA", "YEAR"], how="inner")
        df_current_raw_data_for_add = df_current_raw_data_for_add \
            .withColumn("TIME_DIFF", (col('YEAR') - g_year)) \
            .withColumn("WEIGHT", func.when((col('YEAR') > g_year), (col('YEAR') - g_year - 0.5)).
                        otherwise(col('YEAR') * (-1) + g_year))

        # 当前年与(当前年+1)的增长率所在列的index
        base_index = g_year - min(years) + min(growth_rate_index)
        df_current_raw_data_for_add = df_current_raw_data_for_add.withColumn("SALES_BK", col('SALES'))

        # 为补数计算增长率
        df_current_raw_data_for_add = df_current_raw_data_for_add \
            .withColumn("MIN_INDEX", func.when((col('YEAR') < g_year), (col('TIME_DIFF') + base_index)).
                        otherwise(base_index)) \
            .withColumn("MAX_INDEX", func.when((col('YEAR') < g_year), (base_index - 1)).
                        otherwise(col('TIME_DIFF') + base_index - 1)) \
            .withColumn("TOTAL_GR", func.lit(1))

        # 多年有数的会对增长率进行累计计算
        for i in growth_rate_index:
            col_name = df_current_raw_data_for_add.columns[i]
            df_current_raw_data_for_add = df_current_raw_data_for_add.withColumn(col_name, func.when(
                (col('MIN_INDEX') > i) | (col('MAX_INDEX') < i), 1).
                                                                                 otherwise(
                df_current_raw_data_for_add[col_name]))
            df_current_raw_data_for_add = df_current_raw_data_for_add.withColumn(col_name,
                                                                                 func.when(col('YEAR') > g_year,
                                                                                           col(col_name) ** (-1)).
                                                                                 otherwise(df_current_raw_data_for_add[
                                                                                               col_name]))
            df_current_raw_data_for_add = df_current_raw_data_for_add.withColumn("TOTAL_GR",
                                                                                 col('TOTAL_GR') * col(col_name))

        df_current_raw_data_for_add = df_current_raw_data_for_add.withColumn("FINAL_GR", func.when(col('TOTAL_GR') < 2,
                                                                                                   col('TOTAL_GR')).
                                                                             otherwise(2))

        # 为当前年的缺失数据补数：根据增长率计算 SALES，匹配 price，计算 UNITS=SALES/price
        df_current_adding_data = df_current_raw_data_for_add \
            .withColumn("SALES", col('SALES') * col('FINAL_GR')) \
            .withColumn("YEAR", func.lit(g_year))
        df_current_adding_data = df_current_adding_data.withColumn("YEAR_MONTH", col('YEAR') * 100 + col('MONTH'))
        df_current_adding_data = df_current_adding_data.withColumn("YEAR_MONTH", col("YEAR_MONTH").cast(DoubleType()))

        df_current_adding_data = df_current_adding_data.withColumnRenamed("CITYGROUP", "CITY_TIER") \
            .join(df_price, on=["MIN_STD", "YEAR_MONTH", "CITY_TIER"], how="inner") \
            .join(df_price_city, on=["MIN_STD", "YEAR_MONTH", "CITY", "PROVINCE"], how="left")

        df_current_adding_data = df_current_adding_data.withColumn('PRICE', func.when(col('PRICE_CITY').isNull(),
                                                                                      col('PRICE_TIER')) \
                                                                   .otherwise(col('PRICE_CITY')))

        df_current_adding_data = df_current_adding_data.withColumn("UNITS", func.when(col('SALES') == 0, 0).
                                                                   otherwise(col('SALES') / col('PRICE'))) \
            .na.fill({'UNITS': 0})

        return df_current_adding_data, df_original_range

    # %%
    logger.debug('补数')
    # 2. 执行函数 addDate, 月更新每月分别补数，每次补数1个月
    if g_if_add_data == "True":
        df_raw_data_month = df_raw_data.where(col('MONTH') == g_month)
        df_growth_rate_month = df_growth_rate.where(df_growth_rate.MONTH_FOR_ADD == g_month)

        # 补数：addDate
        df_adding_data_monthly = addDate(df_raw_data_month, df_growth_rate_month)[0]

        # 输出
        df_adding_data_monthly = df_adding_data_monthly.repartition(1)
        df_adding_data_monthly.write.format("parquet") \
            .mode("overwrite").save(p_adding_data)

        df_adding_data = spark.read.parquet(p_adding_data)

    # %%
    # 3. 合并补数部分和原始部分:: 只有当前年当前月的结果
    if g_if_add_data == "True":
        df_raw_data_adding = (df_raw_data.withColumn("ADD_FLAG", func.lit(0))) \
            .union(df_adding_data.withColumn("ADD_FLAG", func.lit(1)).select(df_raw_data.columns + ["ADD_FLAG"]))
    else:
        df_raw_data_adding = df_raw_data.withColumn("ADD_FLAG", func.lit(0))

    df_raw_data_adding_final = df_raw_data_adding \
        .where((col('YEAR') == g_year) & (col('MONTH') == g_month))

    # %%
    # =========== 输出 =============
    df_raw_data_adding_final = df_raw_data_adding_final.repartition(2)
    df_raw_data_adding_final.write.format("parquet") \
        .mode("overwrite").save(p_raw_data_adding_final)

    logger.debug("输出 raw_data_adding_final：" + p_raw_data_adding_final)

    logger.debug('数据执行-Finish')

    # %%
    # check = spark.read.parquet('s3a://ph-max-auto/v0.0.1-2020-06-08/贝达/202012_test/raw_data_adding_final/')
    # check.where(col('Year')==2020).where(col('Month')==12).groupby('add_flag').agg(func.sum('Sales'), func.sum('Units')).show()

    # %%
    # df_raw_data_adding_final.groupby('ADD_FLAG').agg(func.sum('SALES'), func.sum('UNITS')).show()
    # df_raw_data_adding_final.show(1, vertical=True)

    # %%
    # df_data_old = spark.read.parquet("s3a://ph-max-auto/2020-08-11/Max/refactor/runs/max_test_beida_202012_bk/data_adding_monthly/raw_data_adding_final")
    # print (df_data_old.distinct().count() )
    # df_data_old.select("YEAR").distinct().show()

    # df_data_old = df_data_old.withColumnRenamed("SALES", "SALES_OLD")\
    #                             .withColumnRenamed("UNITS", "UNITS_OLD").distinct()

    # compare = df_raw_data_adding_final.join( df_data_old, on=["PHA", "ID", "MIN_STD", "YEAR", "MONTH", "PROVINCE", "CITY", "MOLECULE_STD" ] ,how="inner")
    # print(df_raw_data_adding_final.count(), df_data_old.count(), compare.count() )
    # compare.withColumn("Error", compare["SALES"]- compare["SALES_OLD"] ).select("Error").distinct().collect()
    # compare.withColumn("Error", compare["UNITS"]- compare["UNITS_OLD"] ).select("Error").distinct().collect()

    ###### 有些会存在 province 和 city 为 NULL的情况
    # compare = df_raw_data_adding_final.join( df_data_old, on=["PHA", "ID", "MIN_STD", "YEAR", "MONTH", "PROVINCE", "CITY", "MOLECULE_STD" ] ,how="anti")
    # compare.show()




import os
import subprocess

from phcli.ph_max_auto import define_value as dv
from phcli.ph_max_auto.ph_config.phconfig.phconfig import PhYAMLConfig


class PhRTBase(object):
    """
    Runtime Base Class
    """

    def table_driver_runtime_main_code(self, runtime):
        table = {
            "python3": "phmain.py",
            "r": "phmain.R"
        }
        return table[runtime.strip('\"')]

    def table_driver_runtime_binary(self, runtime):
        table = {
            "bash": "/bin/bash",
            "python3": "python3",
            "r": "Rscript",
        }
        return table[runtime]

    def create_phconf_file(self, path, **kwargs):
        f_lines = self.phs3.open_object_by_lines(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + dv.TEMPLATE_PHCONF_FILE)
        with open(path + "/phconf.yaml", "w") as file:
            for line in f_lines:
                line = line + "\n"
                line = line.replace("$name", kwargs['name']) \
                    .replace("$runtime", kwargs['runtime']) \
                    .replace("$command", kwargs['command']) \
                    .replace("$timeout", str(kwargs['timeout'])) \
                    .replace("$code", self.table_driver_runtime_main_code(kwargs['runtime'])) \
                    .replace("$input", kwargs['input_str']) \
                    .replace("$output", kwargs['output_str'])
                file.write(line)

    def yaml2args(self, path):
        config = PhYAMLConfig(path)
        config.load_yaml()

        f = open(path + "/args.properties", "w")
        for arg in config.spec.containers.args:
            if arg.value != "":
                f.write("--" + arg.key + "\n")
                f.write(str(arg.value) + "\n")

        for output in config.spec.containers.outputs:
            if output.value != "":
                f.write("--" + output.key + "\n")
                f.write(str(output.value) + "\n")
        f.close()

    def create(self, **kwargs):
        raise NotImplementedError

    def table_driver_command_run(self, command):
        table = {
            'submit': self.submit_run,
            'script': self.script_run,
        }
        return table[command]

    def submit_run(self, **kwargs):
        access_key = os.getenv("AWS_ACCESS_KEY_ID", 'NULL_AWS_ACCESS_KEY_ID')
        secret_key = os.getenv("AWS_SECRET_ACCESS_KEY", 'NULL_AWS_SECRET_ACCESS_KEY')
        current_user = os.getenv("HADOOP_PROXY_USER", 'airflow')

        cmd_arr = ["spark-submit",
                   "--master", "yarn",
                   "--deploy-mode", "cluster",
                   "--name", self.job_id,
                   "--proxy-user", current_user,
                   "--queue", 'airflow']

        conf_map = {
            "spark.driver.cores": "1",
            "spark.driver.memory": "4g",
            "spark.executor.cores": "1",
            "spark.executor.memory": "4g",
            "spark.executor.instances": "1",
            "spark.driver.extraJavaOptions": "-Dfile.encoding=UTF-8 "
                                             "-Dsun.jnu.encoding=UTF-8 "
                                             "-Dcom.amazonaws.services.s3.enableV4",
            "spark.executor.extraJavaOptions": "-Dfile.encoding=UTF-8 "
                                               "-Dsun.jnu.encoding=UTF-8 "
                                               "-Dcom.amazonaws.services.s3.enableV4",
            "spark.hadoop.fs.s3a.impl": "org.apache.hadoop.fs.s3a.S3AFileSystem",
            "spark.hadoop.fs.s3a.access.key": access_key,
            "spark.hadoop.fs.s3a.secret.key": secret_key,
            "spark.hadoop.fs.s3a.endpoint": "s3.cn-northwest-1.amazonaws.com.cn"
        }
        conf_map.update(kwargs['submit_conf'])
        conf_map.update(dict([(k.lstrip("CONF__"), v) for k, v in self.context.items() if k.startswith('CONF__')]))
        conf_map = [('--conf', k + '=' + v) for k, v in conf_map.items()]
        cmd_arr += [j for i in conf_map for j in i]

        other_map = {}
        other_map.update(dict([(k.lstrip("OTHER__"), v) for k, v in self.context.items() if k.startswith('OTHER__')]))
        other_map = [('--'+k, v) for k, v in other_map.items()]
        cmd_arr += [j for i in other_map for j in i]

        file_map = kwargs['submit_file']
        file_map = [('--'+k, v) for k, v in file_map.items()]
        cmd_arr += [j for i in file_map for j in i]

        cmd_arr += [kwargs['submit_main']]

        cmd_arr += ['--owner', self.owner]
        cmd_arr += ['--dag_name', self.dag_name]
        cmd_arr += ['--run_id', self.run_id]
        cmd_arr += ['--job_full_name', self.job_full_name]
        cmd_arr += ['--job_id', self.job_id]

        # dag_run 优先 phconf 默认参数
        default_args = self.phs3.open_object_by_lines(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + self.s3_job_path + "/args.properties")
        must_args = [arg.strip() for arg in dv.PRESET_MUST_ARGS.split(",")]
        cur_key = ""
        for it in [arg for arg in default_args if arg]:
            # 如果是 key，记录这个key
            if it[0:2] == "--":
                cur_key = it[2:]
                # 必须参数，不使用用户的配置，用系统注入的
                if it[2:] in must_args:
                    continue
                cmd_arr.append(it)
            else:
                # 必须参数的 value 不处理
                if cur_key in must_args:
                    continue
                if cur_key in self.args.keys():
                    it = self.args[cur_key]
                if it:
                    cmd_arr.append(it)

        return subprocess.check_output(cmd_arr, timeout=float(self.timeout) * 60)

    def script_run(self, **kwargs):
        cmd_arr = []
        cmd_arr += kwargs['entrypoint']

        cmd_arr += ['--owner', self.owner]
        cmd_arr += ['--dag_name', self.dag_name]
        cmd_arr += ['--run_id', self.run_id]
        cmd_arr += ['--job_full_name', self.job_full_name]
        cmd_arr += ['--job_id', self.job_id]

        # dag_run 优先 phconf 默认参数
        default_args = self.phs3.open_object_by_lines(dv.TEMPLATE_BUCKET, dv.CLI_VERSION + self.s3_job_path + "/args.properties")
        must_args = [arg.strip() for arg in dv.PRESET_MUST_ARGS.split(",")]
        cur_key = ""
        for it in [arg for arg in default_args if arg]:
            # 如果是 key，记录这个key
            if it[0:2] == "--":
                cur_key = it[2:]
                # 必须参数，不使用用户的配置，用系统注入的
                if it[2:] in must_args:
                    continue
                cmd_arr.append(it)
            else:
                # 必须参数的 value 不处理
                if cur_key in must_args:
                    continue
                if cur_key in self.args.keys():
                    it = self.args[cur_key]
                if it:
                    cmd_arr.append(it)

        return subprocess.check_output(cmd_arr, timeout=float(self.timeout) * 60)

    def online_run(self, **kwargs):
        self.table_driver_command_run(self.command)()



def mapping():
    return [
        {
            "col_name": "COMPANY",
            "col_desc": "数据公司",
            "candidate": [],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "SOURCE",
            "col_desc": "数据来源",
            "candidate": [],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "TAG",
            "col_desc": "文件标识",
            "candidate": ['_TAG'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "PROVINCE_NAME",
            "col_desc": "省份名",
            "candidate": ['省', "省份", "省/自治区/直辖市"],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "CITY_NAME",
            "col_desc": "城市名",
            "candidate": ['城市'],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "PREFECTURE_NAME",
            "col_desc": "区县名",
            "candidate": [],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "YEAR",
            "col_desc": "年份",
            "candidate": ['年'],
            "type": "Integer",
            "not_null": True,
        },
        {
            "col_name": "QUARTER",
            "col_desc": "季度",
            "candidate": ['季度'],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "MONTH",
            "col_desc": "月份",
            "candidate": ['月', "月份"],
            "type": "Integer",
            "not_null": True,
        },
        {
            "col_name": "HOSP_NAME",
            "col_desc": "医院名",
            "candidate": [],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "HOSP_CODE",
            "col_desc": "医院编码",
            "candidate": ['医院编码'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "HOSP_LEVEL",
            "col_desc": "医院等级",
            "candidate": [],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "ATC",
            "col_desc": "ATC编码",
            "candidate": ['ATC码'],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "MOLE_NAME",
            "col_desc": "分子名",
            "candidate": ['通用名', "药品名称"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "PRODUCT_NAME",
            "col_desc": "商品名",
            "candidate": ['药品商品名', '商品名'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "SPEC",
            "col_desc": "规格",
            "candidate": ['规格', "统一规格"],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "DOSAGE",
            "col_desc": "剂型",
            "candidate": ['剂型'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "PACK_QTY",
            "col_desc": "包装数量",
            "candidate": ['包装数量', '包装规格'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "SALES_QTY_GRAIN",
            "col_desc": "粒度销量",
            "candidate": ['数量（支/片）', '最小制剂单位数量', '最小制剂单位包装数量'],
            "type": "Double",
            "not_null": True,
        },
        {
            "col_name": "SALES_QTY_BOX",
            "col_desc": "盒装销量",
            "candidate": [],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "SALES_QTY_TAG",
            "col_desc": r"销量标识(GRAIN \ BOX \ FULL)",
            "candidate": [],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "SALES_VALUE",
            "col_desc": "销售额",
            "candidate": ['金额（元）', '金额', "金额(元)", "金额_元_"],
            "type": "Double",
            "not_null": True,
        },
        {
            "col_name": "DELIVERY_WAY",
            "col_desc": "给药途径",
            "candidate": ['途径', '给药途径'],
            "type": "String",
            "not_null": False,
        },
        {
            "col_name": "MANUFACTURER_NAME",
            "col_desc": "生产厂商",
            "candidate": ['生产企业', '生产厂家'],
            "type": "String",
            "not_null": True,
        },
        {
            "col_name": "MKT",
            "col_desc": "所属市场",
            "candidate": [],
            "type": "String",
            "not_null": False,
        },
    ]



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This is job template for Pharbers Max Job
"""
import base64
from datetime import datetime
from phcli.ph_db.ph_pg import PhPg
from phcli.ph_logs.ph_logs import phs3logger
from phcli.ph_max_auto.ph_models.asset import Asset


def execute(**kwargs):
    """
        please input your code below
        get spark session: spark = kwargs["spark"]()
    """
    owner = kwargs.pop('owner', None)
    run_id = kwargs.pop('run_id', None)
    job_id = kwargs.pop('job_id', None)

    logger = phs3logger(job_id)

    phjobs = {}

    # merge phjobs 和 kwargs 中的参数信息
    for job_name, job in phjobs.items():
        for input_name, input_path in job['input'].items():
            if input_name in kwargs.keys() and kwargs[input_name]:
                job['input'][input_name] = kwargs[input_name]
        for output_name, output_path in job['output'].items():
            if output_name in kwargs.keys() and kwargs[output_name]:
                job['output'][output_name] = kwargs[output_name]

    # 转换成 DAG 邻接表
    dag_node_info_map = {}
    dag_adj_lst = {}
    for job_name, job in phjobs.items():
        for output_name, output_path in job['output'].items():
            dag_node_info_map[output_path] = (output_name, job_name)
            dag_adj_lst[output_path] = dag_adj_lst.get(output_path, [])
            for input_name, input_path in job['input'].items():
                dag_node_info_map[input_path] = (input_name, job_name)
                dag_adj_lst[output_path].append(input_path)

    # 获取最近的 asset 点
    def get_latest_lst_cr(adj_lst, depend_lst):
        """
        从后向前，递归（CR）获取最近的 asset tuple list
        :param adj_lst: 当前节点依赖的节点列表
        :param depend_lst: 依赖当前节点的节点名称列表
        :return: latest_lst 当前节点最近的依赖列表
        """
        result = []
        for adj_name in adj_lst:
            # 如果循环依赖，直接略过
            if adj_name in depend_lst:
                continue

            # 如果自身是 asset，则保留迭代器
            if 's3a://' in adj_name and 'asset' in adj_name:
                result.append(adj_name)

            depend_lst.add(adj_name)
            subs = get_latest_lst_cr(dag_adj_lst.get(adj_name, []), depend_lst)
            result += subs
        return result

    # 取所有 asset output 的 parent
    asset_parents_lst = {}
    for cur_name, adj_lst in dag_adj_lst.items():
        if not cur_name.startswith('s3a://') or 'asset' not in cur_name:
            continue
        parents = list(get_latest_lst_cr(adj_lst, {cur_name}))
        asset_parents_lst[cur_name] = parents

    if not asset_parents_lst:
        logger.warning("没有需要写入的 asset")
        return {}

    pg = PhPg(
        base64.b64decode('cGgtZGItbGFtYmRhLmNuZ2sxamV1cm1udi5yZHMuY24tbm9ydGh3ZXN0LTEuYW1hem9uYXdzLmNvbS5jbgo=').decode('utf8')[:-1],
        base64.b64decode('NTQzMgo=').decode('utf8')[:-1],
        base64.b64decode('cGhhcmJlcnMK').decode('utf8')[:-1],
        base64.b64decode('QWJjZGUxOTYxMjUK').decode('utf8')[:-1],
        db=base64.b64decode('cGhlbnRyeQo=').decode('utf8')[:-1],
    )

    for asset, parents in asset_parents_lst.items():
        logger.info(asset)
        parents_id = []
        for parent in parents:
            logger.info(parent)
            obj = pg.query(Asset(), source=parent)
            if obj:
                obj_id = obj[0].id
            else:
                obj = pg.insert(Asset(name=dag_node_info_map[parent][0], owner=owner, source=parent))
                obj_id = obj.id
            parents_id.append(obj_id)

        obj = pg.query(Asset(), source=asset)
        if obj:
            obj = obj[0]
            obj.owner = owner
            obj.modified = datetime.now()
            pg.update(obj)
        else:
            pg.insert(Asset(name=dag_node_info_map[asset][0], owner=owner, source=asset))

    pg.commit()
    return {}



from phcli.ph_max_auto.ph_hook.get_spark_session import get_spark_session_func
from phcli.ph_max_auto.ph_hook.get_abs_path import get_result_path
from phcli.ph_max_auto.ph_hook.get_abs_path import get_depends_path
from phcli.ph_max_auto.ph_hook.lineage import lineage
from phcli.ph_max_auto.ph_hook.copy_asset_data import copy_asset_data
from phcli.ph_max_auto.ph_hook.ssm_options import get_args_from_ssm
from phcli.ph_max_auto.ph_hook.ssm_options import delete_args_from_ssm


def exec_before(*args, **kwargs):
    name = kwargs.get('name', None)
    job_id = kwargs.get('job_id', name)

    spark_func = get_spark_session_func(job_id)
    result_path_prefix = get_result_path(kwargs)
    depends_path = get_depends_path(kwargs)
    step_args = get_args_from_ssm(kwargs)

    return {
        'spark': spark_func,
        'result_path_prefix': result_path_prefix,
        'depends_path': depends_path
    }.update(step_args)


def exec_after(*args, **kwargs):
    owner = kwargs.get('owner', None)
    run_id = kwargs.get('run_id', None)
    job_id = kwargs.get('job_id', None)

    delete_args_from_ssm(kwargs)
    lineage(job_id, kwargs)
    copy_asset_data(kwargs)

    return kwargs






# -*- coding: utf-8 -*-

from pypinyin import lazy_pinyin


class MappingFactory(object):
    """
    匹配规则的生成工厂
    """

    def get_specific_mapping(self, source, company):
        """
        根据源和公司获取特定的匹配规则

        :param source: 清洗的元数据类型
        :param company: 清洗的公司名称

        :return: [dict] 返回指定的匹配规则
        """
        source = source.replace("&", "_")
        company = "".join(lazy_pinyin(company))
        ipt_module = __import__('phcli.ph_data_clean.mapping.%s_%s' % (source.lower(), company.lower()))
        ph_data_clean_pkg = getattr(ipt_module, 'ph_data_clean')
        mapping_pkg = getattr(ph_data_clean_pkg, 'mapping')
        mapping_file = getattr(mapping_pkg, '%s_%s' % (source.lower(), company.lower()))
        mapping_table = getattr(mapping_file, 'mapping')
        return mapping_table()




# -*- coding: utf-8 -*-

import click
from phcli.define_value import CLI_CLIENT_VERSION
from phcli.ph_max_auto.phcommand import maxauto
from phcli.ph_logs.__main__ import main as logs_main
from phcli.ph_lmd.__main__ import main as phlam_main
from phcli.ph_admin.__main__ import main as phadmin_main
from phcli.ph_sql.ph_hive.__main__ import main as phhive_main
from phcli.ph_data_clean.__main__ import main as clean_main
from phcli.ph_storage.back_up.__main__ import main as hdfs_back_up_main
from phcli.ph_storage.clean.__main__ import main as hdfs_clean_main
from phcli.ph_dag.__main__ import main as dag_main
from phcli.ph_aws_cli.__main__ import  main as awsinit_main

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(context_settings=CONTEXT_SETTINGS)
def phcli():
    """
    Pharbers Command Line Interface.
    """


@phcli.command("version", short_help="打印版本")
def version():
    click.secho("v"+CLI_CLIENT_VERSION, fg='green', blink=True, bold=True)


phcli.add_command(logs_main)
phcli.add_command(maxauto)
phcli.add_command(phlam_main)
phcli.add_command(phadmin_main)
phcli.add_command(phhive_main)
phcli.add_command(clean_main)
phcli.add_command(hdfs_back_up_main)
phcli.add_command(hdfs_clean_main)
phcli.add_command(dag_main)
phcli.add_command(awsinit_main)

if __name__ == '__main__':
    phcli()



import os
import pytest
from phcli.ph_lmd.model.ph_lambda import PhLambda


@pytest.mark.skip("skip")
def test_ph_lambda_package_python():
    args = {
        "runtime": "python",
        "code_path": "./phlmd/",
        "package_name": "test_ph_lambda_package_python.zip",
    }

    PhLambda().package(args)
    assert os.path.exists(args["package_name"])
    os.remove("test_ph_lambda_package_python.zip")


@pytest.mark.skip("skip")
def test_ph_lambda_package_nodejs():
    pass


@pytest.mark.skip(reason='Deprecation')
def test_ph_lambda_create_local():
    args = {
        "name": "test_ph_lambda_create",
        "version": "",
        "runtime": "python3.8,python3.6",
        "lambda_path": "file/ph_lmd/python-lambda-example-code.zip",
        "lambda_handler": "hello_world.app.lambda_handler",
        "lambda_layers": "test_ph_layer_create",
        "lambda_desc": "test_ph_lambda_create_local 单元测试",
        "vpc_config": {
            "SubnetIds": [
                'subnet-0260eab5acd58bc53',
                'subnet-0e3daa88acef9b136',
            ],
            "SecurityGroupIds": [
                'sg-058404c2ad02dcbb9',
                'sg-09f5205a1194149ab'
            ]
        },
        "lambda_timeout": 50,
        "lambda_memory_size": 128,
        "lambda_env": {'TEST': 'test'},
        "lambda_tag": {"language": "python"},
    }
    assert PhLambda().create(args) != {}


def test_ph_lambda_create_s3():
    args = {
        "name": "test_ph_lambda_create",
        "version": "",
        "runtime": "python3.8,python3.6",
        "lambda_path": "s3://ph-platform/2020-08-10/functions/python/test_ph_lambda_create/python-lambda-example-code.zip",
        "lambda_handler": "hello_world.app.lambda_handler",
        "lambda_layers": "test_ph_layer_create",
        "lambda_desc": "test_ph_lambda_create_local 单元测试",
        "vpc_config": {
            "SubnetIds": [
                'subnet-0260eab5acd58bc53',
                'subnet-0e3daa88acef9b136',
            ],
            "SecurityGroupIds": [
                'sg-058404c2ad02dcbb9',
                'sg-09f5205a1194149ab'
            ]
        },
        "lambda_timeout": 50,
        "lambda_memory_size": 128,
        "lambda_env": {'TEST': 'test'},
        "lambda_tag": {"language": "python"},
    }
    assert PhLambda().create(args) != {}


def test_ph_lambda_lists():
    args = {}
    assert PhLambda().lists(args) != {}


def test_ph_lambda_get():
    args = {
        "name": "test_ph_lambda_create",
    }
    assert PhLambda().get(args) != {}


def test_ph_lambda_update():
    args = {
        "name": "test_ph_lambda_create",
        "version": "",
        "runtime": "python3.8,python3.6",
        "lambda_path": "s3://ph-platform/2020-08-10/functions/python/test_ph_lambda_create/python-lambda-example-code.zip",
        "lambda_handler": "hello_world.app.lambda_handler",
        "lambda_layers": "test_ph_layer_create:12",
        "lambda_desc": "test_ph_lambda_create_local 单元测试",
        "vpc_config": {
            "SubnetIds": [
                'subnet-0260eab5acd58bc53',
                'subnet-0e3daa88acef9b136',
            ],
            "SecurityGroupIds": [
                'sg-058404c2ad02dcbb9',
                'sg-09f5205a1194149ab'
            ]
        },
        "lambda_timeout": 55,
        "lambda_memory_size": 149,
        "lambda_env": {'TEST': 'test'},
        "lambda_tag": {"language": "python"},
    }
    assert PhLambda().update(args) != {}


def test_ph_lambda_stop():
    args = {
        "name": "test_ph_lambda_create",
    }
    assert PhLambda().stop(args) != {}


def test_ph_lambda_start():
    args = {
        "name": "test_ph_lambda_create",
    }
    assert PhLambda().start(args) != {}


@pytest.mark.skip(reason='Deprecation')
def test_ph_lambda_delete_local():
    args = {
        "name": "test_ph_lambda_create_local",
    }
    assert PhLambda().delete(args) != {}


@pytest.mark.skip(reason='Used for test_ph_gateway Test')
def test_ph_lambda_delete_s3():
    args = {
        "name": "test_ph_lambda_create",
    }
    assert PhLambda().delete(args) != {}



# -*- coding: utf-8 -*-
# !/usr/bin/python3


import click
from phcli.ph_storage.model.hdfs_storage import PhHdfsStorage
from phcli.ph_storage.model.local_storage import PhLocalStorage
from phcli.ph_storage.model.s3_storage import PhS3Storage


@click.command("hdfsbackup", short_help='HDFS数据备份到S3')
@click.argument("paths")
def main(paths):
    if paths is None or paths == "{}":
        paths = ["/logs/yarnLogs", "/logs/sparkLogs"]

    if not isinstance(paths, list):
        paths = paths.replace('[', '').replace(']', '').split(',')

    hdfs_ins = PhHdfsStorage(PhLocalStorage(), PhS3Storage())
    hdfs_ins.back_up(paths)


if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-

import os
import yaml
from phcli.ph_data_clean.util.yaml_utils import load_by_file

LOCAL_CACHE_DIR = r'../../../file/ph_data_clean/s3_primitive_data/'
TEST_CACHE_DIR = r'../../../file/ph_data_clean/s3_test_data/'


def get_s3_valid_data(s3_data):
    """
    筛选出s3非空数据

    :param s3_data: 爬取的s3所有数据: type = list
    :return: s3_valid_data: 爬取的s3非空数据: type = list
             null_data_name: 检测出的测试无法使用的空数据路径和sheet name: type = list
    """
    null_data_lst = []
    null_data_name = []
    s3_valid_data = []
    ignore = False

    for data in s3_data:
        for values in data['data'].values():
            if len(values.keys()) <= 1:
                ignore = True
        if len(data['data']) <= 9 or ignore == True:
            null_data_name.append(data['file'])
            # null_data_name.append(data['sheet'])
            null_data_lst.append(data)
            continue
        flag = True
        for key in data['data'].keys():
            if not flag:
                continue
            if 'Unnamed' in key.split(':')[0]:
                null_data_name.append(data['file'])
                null_data_name.append(data['sheet'])
                null_data_lst.append(data)
                flag = False
                continue

    for data in s3_data:
        if data not in null_data_lst:
            s3_valid_data.append(data)

    return s3_valid_data, null_data_name


def get_test_data(s3_valid_data):
    """
    生成可用测试数据

    :param s3_valid_data: 爬取的s3非空数据 type = list
    :return: test_data_lst: 最终测试数据 type = list
    """
    test_data_lst = []
    for data in s3_valid_data:
        for i in (0, 1):
            test_data = {'data': {},
                         'metadata': {'fileName': data['file'],
                                      'providers': [data['company'], data['source']],
                                      'sheetName': data['sheet']}}
            for key in data['data'].keys():
                test_data['data'][key] = data['data'][key][i]

            # 加上tag
            test_data['data']['_Tag'] = 'yes'

            test_data_str = str(test_data['data'])
            test_data['data'] = test_data_str

            meta_data_str = str(test_data['metadata'])
            test_data['metadata'] = meta_data_str

            test_data_lst.append(test_data)
    return test_data_lst


def append_test_data(test_data_lst):
    """
    把可用数据写入指定名称的yaml文件

    :param test_data_lst: 拆开的可用数据
    """
    if sub.split('.')[0] + '-test.yaml' not in os.listdir(TEST_CACHE_DIR):
        with open(test_file, 'a', encoding='UTF-8') as file:
            yaml.dump(test_data_lst, file, default_flow_style=False, encoding='utf-8', allow_unicode=True)


if __name__ == '__main__':
    #     for sub in os.listdir(LOCAL_CACHE_DIR):
    sub = 'CHC-Servier.yaml'
    file = LOCAL_CACHE_DIR + sub
    print('筛选：')
    print(file)
    test_file = TEST_CACHE_DIR + sub.split('.')[0] + '-test.yaml'
    print('存入：')
    print(test_file)
    file_lst = load_by_file(file)
    s3_valid_data, null_data_name = get_s3_valid_data(file_lst)
    test_data_lst = get_test_data(s3_valid_data)
    append_test_data(test_data_lst)
    print('finish')



# -*- coding: utf-8 -*-
from phcli.define_value import *

CLI_VERSION = "2020-11-11"

ASSUME_ROLE_ARN = 'YXJuOmF3cy1jbjppYW06OjQ0NDYwMzgwMzkwNDpyb2xlL1BoLUNsaS1NYXhBdXRv'
ASSUME_ROLE_EXTERNAL_ID = 'Ph-Cli-MaxAuto'

TEMPLATE_BUCKET = "ph-platform"
DAGS_S3_BUCKET = 's3fs-ph-airflow'
DAGS_S3_PREV_PATH = 'airflow/dags/'
DAGS_S3_PHJOBS_PATH = '/jobs/python/phcli/'

ENV_WORKSPACE_KEY = 'PH_WORKSPACE'
ENV_WORKSPACE_DEFAULT = '.'
ENV_CUR_PROJ_KEY = 'PH_CUR_PROJ'
ENV_CUR_PROJ_DEFAULT = '.'
ENV_CUR_IDE_KEY = 'PH_CUR_IDE'
ENV_CUR_IDE_DEFAULT = 'c9'
ENV_CUR_RUNTIME_KEY = 'PH_CUR_RUNTIME'
ENV_CUR_RUNTIME_DEFAULT = 'python3'

TEMPLATE_PHCONF_FILE = "/template/python/phcli/maxauto/phconf-20210104.yaml"
TEMPLATE_PHJOB_FILE_PY = "/template/python/phcli/maxauto/phjob-20210104.tmp"
TEMPLATE_PHJOB_FILE_R = "/template/python/phcli/maxauto/phjob-r-20210122.tmp"
TEMPLATE_PHMAIN_FILE_PY = "/template/python/phcli/maxauto/phmain-20210104.tmp"
TEMPLATE_PHMAIN_FILE_R = "/template/python/phcli/maxauto/phmain-r-20210122.tmp"

TEMPLATE_JUPYTER_PYTHON_FILE = '/template/python/phcli/maxauto/phJupyterPython-20210322.json'
TEMPLATE_JUPYTER_R_FILE = '/template/python/phcli/maxauto/phJupyterR-20210122.json'

TEMPLATE_PHDAG_FILE = "/template/python/phcli/maxauto/phdag-20210104.yaml"
TEMPLATE_PHGRAPHTEMP_FILE = "/template/python/phcli/maxauto/phgraphtemp-20210104.tmp"
TEMPLATE_PHDAGJOB_FILE = "/template/python/phcli/maxauto/phDagJob-20210530.tmp"

DEFAULT_RESULT_PATH_FORMAT_STR = "s3://{bucket_name}/{version}/{dag_name}/"
DEFAULT_RESULT_PATH_BUCKET = "ph-max-auto"
DEFAULT_RESULT_PATH_VERSION = "2020-08-11"
DEFAULT_RESULT_PATH_SUFFIX = "refactor/runs"
DEFAULT_ASSET_PATH_FORMAT_STR = "s3://{bucket_name}/{version}/"
DEFAULT_ASSET_PATH_BUCKET = 'ph-max-auto'
DEFAULT_ASSET_PATH_VERSION = "2020-08-11"
DEFAULT_ASSET_PATH_SUFFIX = "asset"


DEFAULT_ROLE_ARN = 'arn:aws-cn:iam::444603803904:role/Pharbers-IoC-Maintainer'
DEFAULT_MACHINE_TYPE = 'STANDARD'
DEFAULT_MACHINE_ARN_SUFFIX = 'arn:aws-cn:states:cn-northwest-1:444603803904:stateMachine:'

PRESET_MUST_ARGS = 'owner, dag_name, run_id, job_full_name, job_id'



import boto3
from enum import Enum
from phcli.ph_aws.aws_root import PhAWS


class PhMetricDataUnit(Enum):
    SECONDS = 'Seconds'
    MCS = 'Microseconds'
    MS = 'Milliseconds'
    BYTES = 'Bytes'
    KB = 'Kilobytes'
    MB = 'Megabytes'
    GB = 'Gigabytes'
    TB = 'Terabytes'
    BITS = 'Bits'
    KBI = 'Kilobits'
    MBI = 'Megabits'
    GBI = 'Gigabits'
    TBI = 'Terabits'
    PERCENT = 'Percent'
    COUNT = "Count"
    BYTES_SECOND = "Bytes/Second"
    KB_SECOND = "Kilobytes/Second"
    MB_SECOND = "Megabytes/Second"
    GB_SECOND = "Gigabytes/Second"
    TB_SECOND = "Terabytes/Second"
    BITS_SECOND = "Bits/Second"
    KBI_SECOND = "Kilobits/Second"
    MBI_SECOND = "Megabits/Second"
    GBI_SECOND = "Gigabits/Second"
    TBI_SECOND = "Terabits/Second"
    COUNT_SECOND = "Count/Second"
    NONE = "None"


class PhMetricData(object):
    def __init__(self, metric_name, dimensions, value, unit: PhMetricDataUnit, storage_resolution=60):
        self.metric_name = metric_name
        self.dimensions = [{"Name": k, "Value": v} for k,v in dimensions.items()]
        self.value = value
        self.unit = unit
        self.storage_resolution = storage_resolution

    def to_dict(self):
        return {
            'MetricName': self.metric_name,
            'Dimensions': self.dimensions,
            'Value': self.value,
            'Unit': self.unit.value,
            'StorageResolution': self.storage_resolution,
        }


class PhCloudWatch(PhAWS):
    def __init__(self, *args, **kwargs):
        self.access_key = kwargs.get('access_key', None)
        self.secret_key = kwargs.get('secret_key', None)
        if self.access_key and self.secret_key:
            self.cw_client = boto3.client('cloudwatch', region_name='cn-northwest-1',
                                          aws_access_key_id=self.access_key,
                                          aws_secret_access_key=self.secret_key)
            self.cw_resource = boto3.resource('cloudwatch', region_name='cn-northwest-1',
                                              aws_access_key_id=self.access_key,
                                              aws_secret_access_key=self.secret_key)
            return

        self.phsts = kwargs.get('phsts', None)
        if self.phsts and self.phsts.credentials:
            self.cw_client = boto3.client('cloudwatch', **self.phsts.get_cred())
            self.cw_resource = boto3.resource('cloudwatch', **self.phsts.get_cred())
            return

        self.cw_client = boto3.client('cloudwatch')
        self.cw_resource = boto3.resource('cloudwatch')

    def put_metric_data(self, namespace, *metricdatas):
        return self.cw_client.put_metric_data(
            Namespace=namespace,
            MetricData=[md.to_dict() for md in metricdatas]
        )


if __name__ == '__main__':
    import base64
    from phcli.ph_aws.ph_sts import PhSts
    from phcli import define_value as dv

    phsts = PhSts().assume_role(
        base64.b64decode(dv.ASSUME_ROLE_ARN).decode(),
        dv.ASSUME_ROLE_EXTERNAL_ID,
    )
    phcw = PhCloudWatch(phsts=phsts)

    response = phcw.put_metric_data("PhApp", PhMetricData("Call", {"User": "alfred", "Type": "cli", "API": "maxauto.delete"}, 1, PhMetricDataUnit.COUNT))
    print(response)
    print(type(response))






# -*- coding: utf-8 -*-

from enum import Enum
from phcli.ph_data_clean.model.clean_result import CleanResult, Tag


class SalesQtyTag(Enum):
    GRAIN = 'GRAIN'
    BOX = 'BOX'
    FULL = 'FULL'


def check_format(*args, **kwargs):
    mapping = args[0]
    raw_data = args[1]
    for k, v in raw_data.items():
        if isinstance(v, str):
            raw_data[k] = v.strip()
    return raw_data


def change_key(*args, **kwargs):
    mapping = args[0]
    raw_data = kwargs['prev']

    # standardise column name
    new_key_name = {}
    for raw_data_key in raw_data.keys():
        old_key = raw_data_key.split("#")[-1].replace('\n', '').strip()  # remove unwanted symbols
        for m in mapping:
            if old_key.lower() in [key.lower() for key in m["candidate"]]:
                new_key = m["col_name"]
                if new_key not in new_key_name:
                    new_key_name[new_key] = raw_data[raw_data_key]  # write new key name into dict

    # create ordered new dict
    final_data = {}
    for m in mapping:
        for n in new_key_name.keys():
            if m["col_name"] == n:
                final_data[m["col_name"]] = new_key_name[n]
            elif m["col_name"] not in final_data.keys():
                final_data[m["col_name"]] = None

    return final_data


def change_year_month(*args, **kwargs):
    final_data = kwargs['prev']
    if final_data:
        input_year = final_data['YEAR']
        input_month = final_data['MONTH']
        output_month = input_month
        output_year = input_year

        flag = True
        if input_year:
            try:
                input_year = int(float(input_year))
                if len(str(input_year)) == 6:
                    output_month = input_year % 100  # month
                    output_year = (input_year - output_month) // 100  # year
                    flag = False
                elif len(str(input_year)) == 8:
                    date = input_year % 100  # date
                    year_month = (input_year - date) // 100  # year+month
                    output_month = year_month % 100  # month
                    output_year = (year_month - output_month) // 100  # year
                    flag = False
                else:
                    output_year = input_year
            except ValueError:
                output_year = input_year

        if flag is True:
            if input_month:
                try:
                    input_month = int(float(input_month))
                    if len(str(input_month)) == 6:
                        output_month = input_month % 100  # month
                        output_year = (input_month - output_month) // 100  # year
                    elif len(str(input_month)) == 8:
                        date = input_month % 100  # date
                        year_month = (input_month - date) // 100  # year+month
                        output_month = year_month % 100  # month
                        output_year = (year_month - output_month) // 100  # year
                    else:
                        output_month = input_month
                        output_year = input_year
                except ValueError:
                    output_month = input_month

        final_data['YEAR'] = output_year
        final_data['MONTH'] = output_month

    return final_data


def change_sales_tag(*args, **kwargs):
    final_data = kwargs['prev']

    if final_data:
        # TODO 整理销量情况
        if final_data['SALES_QTY_GRAIN']:
            final_data['SALES_QTY_BOX'] = final_data['SALES_QTY_GRAIN']
            final_data['SALES_QTY_TAG'] = SalesQtyTag.GRAIN.value
        elif final_data['SALES_QTY_BOX']:
            final_data['SALES_QTY_GRAIN'] = final_data['SALES_QTY_BOX']
            final_data['SALES_QTY_TAG'] = SalesQtyTag.BOX.value

    return final_data


def reformat_int(*args, **kwargs):
    mapping = args[0]
    final_data = kwargs['prev']
    if final_data:
        for m in mapping:
            if (m['type'] == "Integer") and (final_data[m['col_name']]):
                try:
                    final_data[m['col_name']] = int(float(final_data[m['col_name']]))
                except ValueError:
                    final_data[m['col_name']] = final_data[m['col_name']]

    return final_data


def reformat_null(data_type):
    if data_type == "String":
        return ""
    elif data_type == "Double":
        return 0.0
    elif data_type == "Integer":
        return 0
    elif data_type == "Boolean":
        return False


def define_tag_err(*args, **kwargs):
    mapping = args[0]
    raw_data = args[1]
    final_data = kwargs['prev']
    tag_value = Tag.UNDEFINED

    if raw_data == {}:  # 若原始数据为空
        tag_value = Tag.EMPTY_DICT
        error_msg = 'Error message: empty raw_data'
    elif final_data == {}:  # 若最终字典没有内容
        tag_value = Tag.EMPTY_DICT
        error_msg = 'Error message: no mapping found'

    else:
        error_msg_flag = False
        error_msg = 'Error message: column missing-- '
        for maps in mapping:
            # 若某些必须有的列缺失数据
            if (maps['not_null']) and (final_data[maps['col_name']] is None):
                error_msg_flag = True
                tag_value = Tag.MISSING_COL
                error_msg += ' / ' + maps['col_name']
                final_data[maps['col_name']] = reformat_null(data_type=maps['type'])
                # continue

            elif (maps['not_null']) and (final_data[maps['col_name']] in ['', '/']) and (
                    maps['col_name'] != 'PRODUCT_NAME'):
                error_msg_flag = True
                tag_value = Tag.MISSING_COL
                error_msg += ' / rd_err: ' + maps['col_name']
                # continue

            elif (not maps['not_null']) and (final_data[maps['col_name']] is None):
                final_data[maps['col_name']] = reformat_null(data_type=maps['type'])

        if not error_msg_flag:
            tag_value = Tag.SUCCESS
            error_msg = 'Success'

    return CleanResult(data=final_data,
                       metadata={},
                       raw_data=raw_data,
                       tag=tag_value,
                       err_msg=error_msg),



# -*- coding: utf-8 -*-

import boto3
import base64

from phcli.ph_aws.ph_sts import PhSts
from phcli.ph_aws.ph_s3 import PhS3
from phcli.ph_lmd.model.aws_operator import AWSOperator
from phcli.ph_lmd import define_value as dv
from phcli.ph_lmd.runtime.rt_util import get_short_rt, get_rt_inst


class PhLayer(AWSOperator):
    """
    lambda 的依赖层
    """
    phsts = PhSts().assume_role(
        base64.b64decode(dv.ASSUME_ROLE_ARN).decode(),
        dv.ASSUME_ROLE_EXTERNAL_ID,
    )
    phs3 = PhS3(phsts=phsts)

    def package(self, data):
        """
        对 lambda layer 按照 runtime 打包
        :param data:
            :arg runtime 运行时字符串，“python” 或者 “nodejs” 或者 “go”
            :arg lib_path: python 运行时，仅当 is_pipenv = False 时有效，指定 python 的 lib 位置
            :arg package_name 打包的名称
            :arg is_pipenv: 是否使用的 pipenv 构建的项目，默认为 False
        """
        runtime_inst = get_rt_inst(data['runtime'])
        return runtime_inst.pkg_layer(data)

    def create(self, data):
        """
        创建 lambda 的 layer
        :param data:
            :arg name: layer 名字
            :arg version: layer 版本
            :arg layer_path: layer zip 的位置
                            可以是本地（file/ph_lmd/python-lambda-example-layer.zip，会先被传到 S3）或
                            s3 上的文件（s3://ph-platform/2020-08-10/layers/python/test_ph_layer_create/python-lambda-example-layer.zip）
            :arg runtime: layer 适用的运行时，如果多个请使用 “,” 分割
            :arg layer_desc: layer 的描述
        """
        lambda_client = boto3.client('lambda', **self.phsts.get_cred())

        bucket_name, object_name = self.phs3.sync_file_local_to_s3(
            path=data["layer_path"],
            bucket_name=data.get("bucket", dv.DEFAULT_BUCKET),
            dir_name=dv.CLI_VERSION + dv.DEFAULT_LAYER_DIR
                .replace("#runtime#", get_short_rt(data["runtime"]))
                .replace("#name#", data["name"]),
            version=data.get("version", ""),
        )

        response = lambda_client.publish_layer_version(
            LayerName=data["name"],
            Description=data.get("layer_desc", "phcli create " + data["name"] + " layer"),
            Content={
                'S3Bucket': bucket_name,
                'S3Key': object_name,
            },
            CompatibleRuntimes=data["runtime"].split(","),
            LicenseInfo='MIT'
        )

        return response

    def lists(self, data):
        """
        获取所有 layer 实例
        :param data:
            :arg runtime: layer 适用的运行时，只可指定一个【不强制】
            :arg name: layer 名字
        """
        lambda_client = boto3.client('lambda', **self.phsts.get_cred())

        if "name" in data.keys():
            response = lambda_client.list_layer_versions(
                LayerName=data["name"],
            )
        elif "runtime" in data.keys():
            response = lambda_client.list_layers(
                CompatibleRuntime=data["runtime"],
            )
        else:
            response = lambda_client.list_layers()

        return response

    def get(self, data):
        """
        获取指定的 layer 实例
        :param data:
            :arg name: layer 名字可加版本
        """
        lambda_client = boto3.client('lambda', **self.phsts.get_cred())

        response = lambda_client.list_layer_versions(
            LayerName=data["name"].split(":")[0],
        )

        if not response["LayerVersions"]:
            return {}

        version = data["name"].split(":")[1:2]
        if version:
            for layer in response["LayerVersions"]:
                if int(version[0]) == layer["Version"]:
                    response["LayerVersions"] = [layer]
                    break

        return response

    def update(self, data):
        """
        更新 lambda 的 layer，等于 create layer
        """
        return self.create(data)

    def apply(self, data):
        """
        发布或更新 lambda 的 layer, 等于 create layer
        """
        if self.get(data) == {}:
            return self.create(data)
        else:
            return self.update(data)

    def stop(self, data):
        """
        lambda 的 layer 不可停止
        """
        return self.stop.__doc__

    def start(self, data):
        """
        lambda 的 layer 不可启动
        """
        return self.start.__doc__

    def delete(self, data):
        """
        删除 lambda 的 layer
        :param data:
            :arg name: 要删除的 layer 名字
            :arg version: 要删除的 layer 版本
        """
        lambda_client = boto3.client('lambda', **self.phsts.get_cred())

        response = lambda_client.delete_layer_version(
            LayerName=data["name"],
            VersionNumber=data["version"],
        )

        return response



# -*- coding: utf-8 -*-

import urllib.request
import json
import git
import tarfile
import os


GITHUB_API_URL = 'https://api.github.com/users'
USER_NAME = 'PharbersDeveloper'
DOWNLOAD_PATH = '/Users/clock/Downloads/github'
ZIP_NAME = 'PharbersDeveloper.zip'


def get_all_public_repos(user_name, page=1):
    repos_api = 'repos'
    response = urllib.request.urlopen(f'{GITHUB_API_URL}/{user_name}/{repos_api}?page={page}')
    repos_info = json.load(response)
    repos_len = len(repos_info)

    if not repos_len:
        return []

    page_repos = [repo_info['clone_url'] for repo_info in repos_info]
    return page_repos + get_all_public_repos(user_name, page+1)


def download_repo(repo_url, download_path):
    git.Git(download_path).clone(repo_url)


def get_downloaded_repos(download_path):
    return os.listdir(download_path)


def zip_repos(zip_path, zip_name):
    with tarfile.open(zip_name, "w:gz") as tar:
        tar.add(zip_path, arcname=os.path.basename(zip_path))


if __name__ == '__main__':
    all_repos = get_all_public_repos(USER_NAME)
    downloaded_repos = get_downloaded_repos(DOWNLOAD_PATH)

    repos = [repo for repo in set(all_repos) if repo.split('/')[-1].split('.')[0] not in downloaded_repos]

    repos_count = len(repos)
    for index, repo in enumerate(set(repos)):
        print(f'下载第 {index+1}/{repos_count} 个，{repo}')
        download_repo(repo, DOWNLOAD_PATH)
    else:
        print('下载完成')

    zip_repos(DOWNLOAD_PATH, ZIP_NAME)
